#!/usr/bin/python
# Copyright 2016 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Manage clock skew after migration on a Google Compute Engine instance."""

import logging.handlers
import optparse

from google_compute_engine import config_manager
from google_compute_engine import constants
from google_compute_engine import file_utils
from google_compute_engine import logger
from google_compute_engine import metadata_watcher
from google_compute_engine.compat import distro_utils

LOCKFILE = constants.LOCALSTATEDIR + '/lock/google_clock_skew.lock'


class ClockSkewDaemon(object):
  """Responds to drift-token changes."""

  drift_token = 'instance/virtual-clock/drift-token'

  def __init__(self, debug=False):
    """Constructor.

    Args:
      debug: bool, True if debug output should write to the console.
    """
    facility = logging.handlers.SysLogHandler.LOG_DAEMON
    self.logger = logger.Logger(
        name='google-clock-skew', debug=debug, facility=facility)
    self.distro_utils = distro_utils.Utils(debug=debug)
    self.watcher = metadata_watcher.MetadataWatcher(logger=self.logger)
    try:
      with file_utils.LockFile(LOCKFILE):
        self.logger.info('Starting Google Clock Skew daemon.')
        self.watcher.WatchMetadata(
            self.HandleClockSync, metadata_key=self.drift_token,
            recursive=False)
    except (IOError, OSError) as e:
      self.logger.warning(str(e))

  def HandleClockSync(self, response):
    """Called when clock drift token changes.

    Args:
      response: string, the metadata response with the new drift token value.
    """
    self.logger.info('Clock drift token has changed: %s.', response)
    self.distro_utils.HandleClockSync(self.logger)


def main():
  parser = optparse.OptionParser()
  parser.add_option(
      '-d', '--debug', action='store_true', dest='debug',
      help='print debug output to the console.')
  (options, _) = parser.parse_args()
  instance_config = config_manager.ConfigManager()
  if instance_config.GetOptionBool('Daemons', 'clock_skew_daemon'):
    ClockSkewDaemon(debug=bool(options.debug))


if __name__ == '__main__':
  main()



#!/usr/bin/python
# Copyright 2018 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Unittest for ip_forwarding.py module."""

from google_compute_engine.networking.ip_forwarding import ip_forwarding
from google_compute_engine.test_compat import mock
from google_compute_engine.test_compat import unittest


class IpForwardingTest(unittest.TestCase):

  def setUp(self):
    self.mock_logger = mock.Mock()
    self.mock_watcher = mock.Mock()
    self.mock_ip_forwarding_utils = mock.Mock()
    self.mock_setup = mock.create_autospec(ip_forwarding.IpForwarding)
    self.mock_setup.logger = self.mock_logger
    self.mock_setup.ip_forwarding_utils = self.mock_ip_forwarding_utils

  @mock.patch('google_compute_engine.networking.ip_forwarding.ip_forwarding.ip_forwarding_utils')
  @mock.patch('google_compute_engine.networking.ip_forwarding.ip_forwarding.logger')
  def testIpForwarding(self, mock_logger, mock_ip_forwarding_utils):
    mock_logger_instance = mock.Mock()
    mock_logger.Logger.return_value = mock_logger_instance
    mocks = mock.Mock()
    mocks.attach_mock(mock_logger, 'logger')
    mocks.attach_mock(mock_ip_forwarding_utils, 'forwarding')
    with mock.patch.object(ip_forwarding.IpForwarding, 'HandleForwardedIps'):

      ip_forwarding.IpForwarding(proto_id='66', debug=True)
      expected_calls = [
          mock.call.logger.Logger(name=mock.ANY, debug=True, facility=mock.ANY),
          mock.call.forwarding.IpForwardingUtils(
              logger=mock_logger_instance, proto_id='66'),
      ]
      self.assertEqual(mocks.mock_calls, expected_calls)

  def testLogForwardedIpChanges(self):
    ip_forwarding.IpForwarding._LogForwardedIpChanges(
        self.mock_setup, [], [], [], [], '1')
    ip_forwarding.IpForwarding._LogForwardedIpChanges(
        self.mock_setup, ['a'], ['a'], [], [], '2')
    ip_forwarding.IpForwarding._LogForwardedIpChanges(
        self.mock_setup, ['a'], [], [], ['a'], '3')
    ip_forwarding.IpForwarding._LogForwardedIpChanges(
        self.mock_setup, ['a', 'b'], ['b'], [], ['a'], '4')
    ip_forwarding.IpForwarding._LogForwardedIpChanges(
        self.mock_setup, ['a'], ['b'], ['b'], ['a'], '5')
    expected_calls = [
        mock.call.info(mock.ANY, '3', ['a'], None, None, ['a']),
        mock.call.info(mock.ANY, '4', ['a', 'b'], ['b'], None, ['a']),
        mock.call.info(mock.ANY, '5', ['a'], ['b'], ['b'], ['a']),
    ]
    self.assertEqual(self.mock_logger.mock_calls, expected_calls)

  def testAddForwardedIp(self):
    ip_forwarding.IpForwarding._AddForwardedIps(
        self.mock_setup, [], 'interface')
    self.assertEqual(self.mock_ip_forwarding_utils.mock_calls, [])

    ip_forwarding.IpForwarding._AddForwardedIps(
        self.mock_setup, ['a', 'b', 'c'], 'interface')
    expected_calls = [
        mock.call.AddForwardedIp('a', 'interface'),
        mock.call.AddForwardedIp('b', 'interface'),
        mock.call.AddForwardedIp('c', 'interface'),
    ]
    self.assertEqual(self.mock_ip_forwarding_utils.mock_calls, expected_calls)

  def testRemoveForwardedIp(self):
    ip_forwarding.IpForwarding._RemoveForwardedIps(
        self.mock_setup, [], 'interface')
    self.assertEqual(self.mock_ip_forwarding_utils.mock_calls, [])

    ip_forwarding.IpForwarding._RemoveForwardedIps(
        self.mock_setup, ['a', 'b', 'c'], 'interface')
    expected_calls = [
        mock.call.RemoveForwardedIp('a', 'interface'),
        mock.call.RemoveForwardedIp('b', 'interface'),
        mock.call.RemoveForwardedIp('c', 'interface'),
    ]
    self.assertEqual(self.mock_ip_forwarding_utils.mock_calls, expected_calls)

  def testHandleForwardedIps(self):
    configured = ['c', 'c', 'b', 'b', 'a', 'a']
    desired = ['d', 'd', 'c']
    mocks = mock.Mock()
    mocks.attach_mock(self.mock_ip_forwarding_utils, 'forwarding')
    mocks.attach_mock(self.mock_setup, 'setup')
    self.mock_ip_forwarding_utils.ParseForwardedIps.return_value = desired
    self.mock_ip_forwarding_utils.GetForwardedIps.return_value = configured
    forwarded_ips = 'forwarded ips'
    interface_ip = 'interface ip'
    interface = 'interface'
    expected_add = ['d']
    expected_remove = ['a', 'b']

    ip_forwarding.IpForwarding.HandleForwardedIps(
        self.mock_setup, interface, forwarded_ips, interface_ip)
    expected_calls = [
        mock.call.forwarding.ParseForwardedIps(forwarded_ips),
        mock.call.forwarding.GetForwardedIps(interface, interface_ip),
        mock.call.setup._LogForwardedIpChanges(
            configured, desired, expected_add, expected_remove, interface),
        mock.call.setup._AddForwardedIps(expected_add, interface),
        mock.call.setup._RemoveForwardedIps(expected_remove, interface),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)



#!/usr/bin/python
# Copyright 2018 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Unittest for network_daemon.py module."""

from google_compute_engine import network_utils
from google_compute_engine.networking import network_daemon
from google_compute_engine.test_compat import mock
from google_compute_engine.test_compat import unittest


class NetworkDaemonTest(unittest.TestCase):

  def setUp(self):
    self.mock_logger = mock.Mock()
    self.mock_watcher = mock.Mock()
    self.mock_setup = mock.create_autospec(network_daemon.NetworkDaemon)
    self.mock_setup.logger = self.mock_logger
    self.mock_setup.watcher = self.mock_watcher
    self.mock_ip_forwarding = mock.Mock()
    self.mock_network_setup = mock.Mock()
    self.mock_network_utils = mock.Mock()
    self.mock_setup.ip_forwarding = self.mock_ip_forwarding
    self.mock_setup.network_setup = self.mock_network_setup
    self.mock_setup.network_utils = self.mock_network_utils

  @mock.patch('google_compute_engine.networking.network_daemon.ip_forwarding')
  @mock.patch('google_compute_engine.networking.network_daemon.network_setup')
  @mock.patch('google_compute_engine.networking.network_daemon.network_utils')
  @mock.patch('google_compute_engine.networking.network_daemon.metadata_watcher')
  @mock.patch('google_compute_engine.networking.network_daemon.logger')
  @mock.patch('google_compute_engine.networking.network_daemon.file_utils')
  def testNetworkDaemon(
      self, mock_lock, mock_logger, mock_watcher, mock_network_utils,
      mock_network_setup, mock_ip_forwarding):
    mock_logger_instance = mock.Mock()
    mock_logger.Logger.return_value = mock_logger_instance
    mocks = mock.Mock()
    mocks.attach_mock(mock_lock, 'lock')
    mocks.attach_mock(mock_logger, 'logger')
    mocks.attach_mock(mock_network_utils, 'network')
    mocks.attach_mock(mock_ip_forwarding, 'forwarding')
    mocks.attach_mock(mock_network_setup, 'network_setup')
    mocks.attach_mock(mock_watcher, 'watcher')
    metadata_key = network_daemon.NetworkDaemon.network_interface_metadata_key

    with mock.patch.object(
        network_daemon.NetworkDaemon, 'HandleNetworkInterfaces'
    ) as mock_handle:
      network_daemon.NetworkDaemon(
          ip_forwarding_enabled=True,
          proto_id='66',
          ip_aliases=None,
          target_instance_ips=None,
          dhclient_script='x',
          dhcp_command='y',
          network_setup_enabled=True,
          debug=True)
      expected_calls = [
          mock.call.logger.Logger(name=mock.ANY, debug=True, facility=mock.ANY),
          mock.call.forwarding.IpForwarding(proto_id='66', debug=True),
          mock.call.network_setup.NetworkSetup(
              debug=True, dhclient_script='x', dhcp_command='y'),
          mock.call.network.NetworkUtils(logger=mock_logger_instance),
          mock.call.watcher.MetadataWatcher(logger=mock_logger_instance),
          mock.call.lock.LockFile(network_daemon.LOCKFILE),
          mock.call.lock.LockFile().__enter__(),
          mock.call.logger.Logger().info(mock.ANY),
          mock.call.watcher.MetadataWatcher().WatchMetadata(
              mock_handle, metadata_key=metadata_key, recursive=True,
              timeout=mock.ANY),
          mock.call.lock.LockFile().__exit__(None, None, None),
      ]
      self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.networking.network_daemon.ip_forwarding')
  @mock.patch('google_compute_engine.networking.network_daemon.network_setup')
  @mock.patch('google_compute_engine.networking.network_daemon.network_utils')
  @mock.patch('google_compute_engine.networking.network_daemon.metadata_watcher')
  @mock.patch('google_compute_engine.networking.network_daemon.logger')
  @mock.patch('google_compute_engine.networking.network_daemon.file_utils')
  def testNetworkDaemonError(
      self, mock_lock, mock_logger, mock_watcher, mock_network_utils,
      mock_network_setup, mock_ip_forwarding):
    mock_logger_instance = mock.Mock()
    mock_logger.Logger.return_value = mock_logger_instance
    mocks = mock.Mock()
    mocks.attach_mock(mock_lock, 'lock')
    mocks.attach_mock(mock_logger, 'logger')
    mocks.attach_mock(mock_ip_forwarding, 'forwarding')
    mocks.attach_mock(mock_network_setup, 'network_setup')
    mocks.attach_mock(mock_network_utils, 'network')
    mocks.attach_mock(mock_watcher, 'watcher')
    self.mock_setup._ExtractInterfaceMetadata.return_value = []
    mock_lock.LockFile.side_effect = IOError('Test Error')

    with mock.patch.object(
        network_daemon.NetworkDaemon, 'HandleNetworkInterfaces'):
      network_daemon.NetworkDaemon(
          ip_forwarding_enabled=False,
          proto_id='66',
          ip_aliases=None,
          target_instance_ips=None,
          dhclient_script='x',
          dhcp_command='y',
          network_setup_enabled=False,
          debug=True)
      expected_calls = [
          mock.call.logger.Logger(name=mock.ANY, debug=True, facility=mock.ANY),
          mock.call.forwarding.IpForwarding(proto_id='66', debug=True),
          mock.call.network_setup.NetworkSetup(
              debug=True, dhclient_script='x', dhcp_command='y'),
          mock.call.network.NetworkUtils(logger=mock_logger_instance),
          mock.call.watcher.MetadataWatcher(logger=mock_logger_instance),
          mock.call.lock.LockFile(network_daemon.LOCKFILE),
          mock.call.logger.Logger().warning('Test Error'),
      ]
      self.assertEqual(mocks.mock_calls, expected_calls)

  def testHandleNetworkInterfaces(self):
    mocks = mock.Mock()
    mocks.attach_mock(self.mock_ip_forwarding, 'forwarding')
    mocks.attach_mock(self.mock_network_setup, 'network_setup')
    mocks.attach_mock(self.mock_setup, 'setup')
    self.mock_setup.ip_aliases = None
    self.mock_setup.target_instance_ips = None
    self.mock_setup.ip_forwarding_enabled = True
    self.mock_setup.network_setup_enabled = True
    self.mock_setup._ExtractInterfaceMetadata.return_value = [
        network_daemon.NetworkDaemon.NetworkInterface(
            'eth0', forwarded_ips=['a'], ip='1.1.1.1', ipv6=False),
        network_daemon.NetworkDaemon.NetworkInterface('eth1'),
    ]
    result = mock.Mock()

    network_daemon.NetworkDaemon.HandleNetworkInterfaces(
        self.mock_setup, result)
    expected_calls = [
        mock.call.setup._ExtractInterfaceMetadata(result),
        mock.call.network_setup.DisableIpv6(['eth0']),
        mock.call.network_setup.EnableNetworkInterfaces(['eth1']),
        mock.call.forwarding.HandleForwardedIps(
            'eth0', ['a'], '1.1.1.1'),
        mock.call.forwarding.HandleForwardedIps('eth1', None, None),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)

  def testHandleNetworkInterfacesIpv6(self):
    mocks = mock.Mock()
    mocks.attach_mock(self.mock_ip_forwarding, 'forwarding')
    mocks.attach_mock(self.mock_network_setup, 'network_setup')
    mocks.attach_mock(self.mock_setup, 'setup')
    self.mock_setup.ip_aliases = None
    self.mock_setup.target_instance_ips = None
    self.mock_setup.ip_forwarding_enabled = True
    self.mock_setup.network_setup_enabled = True
    self.mock_setup._ExtractInterfaceMetadata.return_value = [
        network_daemon.NetworkDaemon.NetworkInterface(
            'eth0', forwarded_ips=['a'], ip='1.1.1.1', ipv6=True),
    ]
    result = mock.Mock()

    network_daemon.NetworkDaemon.HandleNetworkInterfaces(
        self.mock_setup, result)
    expected_calls = [
        mock.call.setup._ExtractInterfaceMetadata(result),
        mock.call.network_setup.EnableIpv6(['eth0']),
        mock.call.network_setup.EnableNetworkInterfaces([]),
        mock.call.forwarding.HandleForwardedIps(
            'eth0', ['a'], '1.1.1.1'),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)

  def testHandleNetworkInterfacesIpv6Disabled(self):
    mocks = mock.Mock()
    mocks.attach_mock(self.mock_ip_forwarding, 'forwarding')
    mocks.attach_mock(self.mock_network_setup, 'network_setup')
    mocks.attach_mock(self.mock_setup, 'setup')
    self.mock_setup.ip_aliases = None
    self.mock_setup.target_instance_ips = None
    self.mock_setup.ip_forwarding_enabled = True
    self.mock_setup.network_setup_enabled = True
    self.mock_setup._ExtractInterfaceMetadata.return_value = [
        network_daemon.NetworkDaemon.NetworkInterface(
            'eth0', forwarded_ips=['a'], ip='1.1.1.1', ipv6=False),
    ]
    result = mock.Mock()

    network_daemon.NetworkDaemon.HandleNetworkInterfaces(
        self.mock_setup, result)
    expected_calls = [
        mock.call.setup._ExtractInterfaceMetadata(result),
        mock.call.network_setup.DisableIpv6(['eth0']),
        mock.call.network_setup.EnableNetworkInterfaces([]),
        mock.call.forwarding.HandleForwardedIps(
            'eth0', ['a'], '1.1.1.1'),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)

  def testHandleNetworkInterfacesDisabled(self):
    mocks = mock.Mock()
    mocks.attach_mock(self.mock_ip_forwarding, 'forwarding')
    mocks.attach_mock(self.mock_network_setup, 'network_setup')
    mocks.attach_mock(self.mock_setup, 'setup')
    self.mock_setup.ip_aliases = None
    self.mock_setup.target_instance_ips = None
    self.mock_setup.ip_forwarding_enabled = False
    self.mock_setup.network_setup_enabled = False
    self.mock_setup._ExtractInterfaceMetadata.return_value = [
        network_daemon.NetworkDaemon.NetworkInterface('a'),
        network_daemon.NetworkDaemon.NetworkInterface('b'),
    ]
    result = mock.Mock()

    network_daemon.NetworkDaemon.HandleNetworkInterfaces(
        self.mock_setup, result)
    expected_calls = [
        mock.call.setup._ExtractInterfaceMetadata(result),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)

  def testExtractInterfaceMetadata(self):
    self.mock_setup.ip_aliases = True
    self.mock_setup.target_instance_ips = True
    self.mock_setup.network_utils = network_utils.NetworkUtils()
    self.mock_setup.network_utils.interfaces = {
        '1': 'eth0', '2': 'eth1', '3': 'eth2',
    }
    metadata = [
        {
            'mac': '1',
            'forwardedIps': ['a'],
            'dhcpv6Refresh': 1,
        },
        {
            'mac': '2',
            'forwardedIps': ['b'],
            'ipAliases': ['banana'],
            'targetInstanceIps': ['baklava'],
            'ip': '2.2.2.2',
            'dhcpv6Refresh': 2,
        },
        {
            'mac': '3',
            'ipAliases': ['cherry'],
            'targetInstanceIps': ['cake'],
        },
        {
            'mac': '4',
        },
        {
            'forwardedIps': ['d'],
            'ipAliases': ['date'],
            'targetInstanceIps': ['doughnuts'],
        },
    ]
    expected_interfaces = [
        network_daemon.NetworkDaemon.NetworkInterface(
            'eth0', forwarded_ips=['a'], ip=None, ipv6=True),
        network_daemon.NetworkDaemon.NetworkInterface(
            'eth1', forwarded_ips=['b', 'banana', 'baklava'], ip='2.2.2.2',
            ipv6=True),
        network_daemon.NetworkDaemon.NetworkInterface(
            'eth2', forwarded_ips=['cherry', 'cake'], ip=None),
    ]

    actual_interfaces = network_daemon.NetworkDaemon._ExtractInterfaceMetadata(
        self.mock_setup, metadata)
    for actual, expected in zip(actual_interfaces, expected_interfaces):
      self.assertEqual(actual.name, expected.name)
      self.assertEqual(actual.forwarded_ips, expected.forwarded_ips)
      self.assertEqual(actual.ip, expected.ip)
      self.assertEqual(actual.ipv6, expected.ipv6)

  def testExtractInterfaceMetadataWithoutOptions(self):
    self.mock_setup.ip_aliases = None
    self.mock_setup.target_instance_ips = None
    self.mock_setup.network_utils = network_utils.NetworkUtils()
    self.mock_setup.network_utils.interfaces = {
        '1': 'eth0', '2': 'eth1', '3': 'eth2',
    }
    metadata = [
        {
            'mac': '1',
            'forwardedIps': ['a'],
            'dhcpv6Refresh': 1,
        },
        {
            'mac': '2',
            'forwardedIps': ['b'],
            'ipAliases': ['banana'],
            'targetInstanceIps': ['baklava'],
            'ip': '2.2.2.2',
            'dhcpv6Refresh': 2,
        },
        {
            'mac': '3',
            'ipAliases': ['cherry'],
            'targetInstanceIps': ['cake'],
        },
    ]
    expected_interfaces = [
        network_daemon.NetworkDaemon.NetworkInterface(
            'eth0', forwarded_ips=['a'], ip=None, ipv6=True),
        network_daemon.NetworkDaemon.NetworkInterface(
            'eth1', forwarded_ips=['b'], ip='2.2.2.2', ipv6=True),
        network_daemon.NetworkDaemon.NetworkInterface(
            'eth2', forwarded_ips=[], ip=None, ipv6=False),
    ]

    actual_interfaces = network_daemon.NetworkDaemon._ExtractInterfaceMetadata(
        self.mock_setup, metadata)
    for actual, expected in zip(actual_interfaces, expected_interfaces):
      self.assertEqual(actual.name, expected.name)
      self.assertEqual(actual.forwarded_ips, expected.forwarded_ips)
      self.assertEqual(actual.ip, expected.ip)
      self.assertEqual(actual.ipv6, expected.ipv6)



#!/usr/bin/python
# Copyright 2016 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Unittest for ip_forwarding_utils.py module."""

from google_compute_engine.distro_lib import ip_forwarding_utils
from google_compute_engine.test_compat import mock
from google_compute_engine.test_compat import unittest


def _CreateMockProcess(returncode, stdout, stderr):
  mock_process = mock.Mock()
  mock_process.returncode = returncode
  mock_process.communicate.return_value = (stdout, stderr)
  return mock_process


class IpForwardingUtilsIprouteTest(unittest.TestCase):

  def setUp(self):
    self.mock_logger = mock.Mock()
    self.options = {'hello': 'world'}
    with mock.patch(
        'google_compute_engine.distro_lib.ip_forwarding_utils'
        '.subprocess') as mock_subprocess:
      mock_subprocess.Popen.return_value = _CreateMockProcess(
          0, b'out', b'')
      self.mock_utils = ip_forwarding_utils.IpForwardingUtilsIproute(
          self.mock_logger)
    self.mock_utils.proto_id = 'proto'

  def testCreateRouteOptions(self):
    # Default options.
    expected_options = {
        'proto': 'proto',
        'scope': 'host',
    }
    self.assertEqual(self.mock_utils._CreateRouteOptions(), expected_options)

    # Update dictionary when arguments are specified.
    expected_options = {
        'proto': 'proto',
        'scope': 'host',
        'num': 1,
        'string': 'hello world',
    }
    self.assertEqual(
        self.mock_utils._CreateRouteOptions(num=1, string='hello world'),
        expected_options)

    # Update the default options.
    expected_options = {
        'proto': 'test 1',
        'scope': 'test 2',
    }
    self.assertEqual(
        self.mock_utils._CreateRouteOptions(proto='test 1', scope='test 2'),
        expected_options)

  @mock.patch('google_compute_engine.distro_lib.ip_forwarding_utils.subprocess')
  def testRunIpRoute(self, mock_subprocess):
    mock_process = _CreateMockProcess(0, b'out', b'')
    mock_subprocess.Popen.return_value = mock_process
    args = ['foo', 'bar']
    options = {'one': 'two'}

    self.assertEqual(
        self.mock_utils._RunIpRoute(args=args, options=options), 'out')
    command = ['ip', 'route', 'foo', 'bar', 'one', 'two']
    mock_subprocess.Popen.assert_called_once_with(
        command, stdout=mock_subprocess.PIPE, stderr=mock_subprocess.PIPE)
    mock_process.communicate.assert_called_once_with()
    self.mock_logger.warning.assert_not_called()

  @mock.patch('google_compute_engine.distro_lib.ip_forwarding_utils.subprocess')
  def testRunIpRouteReturnCode(self, mock_subprocess):
    mock_process = _CreateMockProcess(1, b'out', b'error\n')
    mock_subprocess.Popen.return_value = mock_process

    self.assertEqual(
        self.mock_utils._RunIpRoute(args=['foo', 'bar'], options=self.options),
        '')
    command = ['ip', 'route', 'foo', 'bar', 'hello', 'world']
    self.mock_logger.warning.assert_called_once_with(
        mock.ANY, command, b'error')

  @mock.patch('google_compute_engine.distro_lib.ip_forwarding_utils.subprocess')
  def testRunIpRouteException(self, mock_subprocess):
    mock_subprocess.Popen.side_effect = OSError('Test Error')

    self.assertEqual(
        self.mock_utils._RunIpRoute(args=['foo', 'bar'], options=self.options),
        '')
    command = ['ip', 'route', 'foo', 'bar', 'hello', 'world']
    self.mock_logger.warning.assert_called_once_with(
        mock.ANY, command, 'Test Error')

  def testParseForwardedIps(self):
    self.assertEqual(self.mock_utils.ParseForwardedIps(None), [])
    self.assertEqual(self.mock_utils.ParseForwardedIps([]), [])
    self.assertEqual(self.mock_utils.ParseForwardedIps([None]), [])
    self.assertEqual(self.mock_utils.ParseForwardedIps(['invalid']), [])
    self.assertEqual(self.mock_utils.ParseForwardedIps(['1a1a1a1']), [])
    self.assertEqual(self.mock_utils.ParseForwardedIps(['1.1.1.1.1']), [])
    self.assertEqual(self.mock_utils.ParseForwardedIps(['1111.1.1.1']), [])
    self.assertEqual(self.mock_utils.ParseForwardedIps(['1.1.1.1111']), [])
    expected_calls = [
        mock.call.warning(mock.ANY, None),
        mock.call.warning(mock.ANY, 'invalid'),
        mock.call.warning(mock.ANY, '1a1a1a1'),
        mock.call.warning(mock.ANY, '1.1.1.1.1'),
        mock.call.warning(mock.ANY, '1111.1.1.1'),
        mock.call.warning(mock.ANY, '1.1.1.1111'),
    ]
    self.assertEqual(self.mock_logger.mock_calls, expected_calls)

  def testParseForwardedIpsComplex(self):
    forwarded_ips = {
        '{{}}\n\"hello\"\n!@#$%^&*()\n\n': False,
        '1111.1.1.1': False,
        '1.1.1.1': True,
        'hello': False,
        '123.123.123.123': True,
        '1.1.1.': False,
        '1.1.1.a': False,
        None: False,
        '1.0.0.0': True,
        '1.1.1.1/1': True,
        '1.1.1.1/11': True,
        '123.123.123.123/1': True,
        '123.123.123.123/123': False,
        '123.123.123.123/a': False,
        '123.123.123.123/': False,
    }
    input_ips = forwarded_ips.keys()
    valid_ips = [ip for ip, valid in forwarded_ips.items() if valid]
    invalid_ips = [ip for ip, valid in forwarded_ips.items() if not valid]

    self.assertEqual(self.mock_utils.ParseForwardedIps(input_ips), valid_ips)
    expected_calls = [mock.call.warning(mock.ANY, ip) for ip in invalid_ips]
    self.assertEqual(self.mock_logger.mock_calls, expected_calls)

  def testParseForwardedIpsSubnet(self):
    forwarded_ips = {
        '1.1.1.1': '1.1.1.1',
        '1.1.1.1/32': '1.1.1.1',
        '1.1.1.1/1': '1.1.1.1/1',
        '1.1.1.1/10': '1.1.1.1/10',
        '1.1.1.1/24': '1.1.1.1/24',
    }
    for ip, value in forwarded_ips.items():
      self.assertEqual(self.mock_utils.ParseForwardedIps([ip]), [value])

  def testGetForwardedIps(self):
    mock_options = mock.Mock()
    mock_options.return_value = self.options
    mock_run = mock.Mock()
    mock_run.return_value = 'a\n b \nlocal c'
    mock_parse = mock.Mock()
    mock_parse.return_value = ['Test']
    self.mock_utils._CreateRouteOptions = mock_options
    self.mock_utils._RunIpRoute = mock_run
    self.mock_utils.ParseForwardedIps = mock_parse

    self.assertEqual(
        self.mock_utils.GetForwardedIps('interface', 'ip'), ['Test'])
    mock_options.assert_called_once_with(dev='interface')
    mock_run.assert_called_once_with(
        args=['ls', 'table', 'local', 'type', 'local'], options=self.options)
    mock_parse.assert_called_once_with(['a', 'b', 'c'])

  def testAddForwardedIp(self):
    mock_options = mock.Mock()
    mock_options.return_value = self.options
    mock_run = mock.Mock()
    self.mock_utils._CreateRouteOptions = mock_options
    self.mock_utils._RunIpRoute = mock_run

    self.mock_utils.AddForwardedIp('1.1.1.1', 'interface')
    mock_options.assert_called_once_with(dev='interface')
    mock_run.assert_called_once_with(
        args=['add', 'to', 'local', '1.1.1.1/32'], options=self.options)

  def testAddIpAlias(self):
    mock_options = mock.Mock()
    mock_options.return_value = self.options
    mock_run = mock.Mock()
    self.mock_utils._CreateRouteOptions = mock_options
    self.mock_utils._RunIpRoute = mock_run

    self.mock_utils.AddForwardedIp('1.1.1.1/24', 'interface')
    mock_options.assert_called_once_with(dev='interface')
    mock_run.assert_called_once_with(
        args=['add', 'to', 'local', '1.1.1.1/24'], options=self.options)

  def testRemoveForwardedIp(self):
    mock_options = mock.Mock()
    mock_options.return_value = self.options
    mock_run = mock.Mock()
    self.mock_utils._CreateRouteOptions = mock_options
    self.mock_utils._RunIpRoute = mock_run

    self.mock_utils.RemoveForwardedIp('1.1.1.1', 'interface')
    mock_options.assert_called_once_with(dev='interface')
    mock_run.assert_called_once_with(
        args=['delete', 'to', 'local', '1.1.1.1/32'], options=self.options)

  def testRemoveAliasIp(self):
    mock_options = mock.Mock()
    mock_options.return_value = self.options
    mock_run = mock.Mock()
    self.mock_utils._CreateRouteOptions = mock_options
    self.mock_utils._RunIpRoute = mock_run

    self.mock_utils.RemoveForwardedIp('1.1.1.1/24', 'interface')
    mock_options.assert_called_once_with(dev='interface')
    mock_run.assert_called_once_with(
        args=['delete', 'to', 'local', '1.1.1.1/24'], options=self.options)


class IpForwardingUtilsIfconfigTest(unittest.TestCase):

  def setUp(self):
    self.mock_logger = mock.Mock()
    with mock.patch(
        'google_compute_engine.distro_lib.ip_forwarding_utils'
        '.subprocess') as mock_subprocess:
      mock_subprocess.Popen.return_value = _CreateMockProcess(
          0, b'out', b'')
      self.mock_utils = ip_forwarding_utils.IpForwardingUtilsIfconfig(
          self.mock_logger)

  @mock.patch('google_compute_engine.distro_lib.ip_forwarding_utils.subprocess')
  def testRunIfconfig(self, mock_subprocess):
    mock_process = _CreateMockProcess(0, b'out', b'')
    mock_subprocess.Popen.return_value = mock_process
    args = ['foo', 'bar']
    options = {'one': 'two'}

    self.assertEqual(
        self.mock_utils._RunIfconfig(args=args, options=options), 'out')
    command = ['ifconfig', 'foo', 'bar', 'one', 'two']
    mock_subprocess.Popen.assert_called_once_with(
        command, stdout=mock_subprocess.PIPE, stderr=mock_subprocess.PIPE)
    mock_process.communicate.assert_called_once_with()
    self.mock_logger.warning.assert_not_called()

  @mock.patch('google_compute_engine.distro_lib.ip_forwarding_utils.subprocess')
  def testRunIfconfigReturnCode(self, mock_subprocess):
    mock_process = _CreateMockProcess(1, b'out', b'error\n')
    mock_subprocess.Popen.return_value = mock_process

    self.assertEqual(
        self.mock_utils._RunIfconfig(args=['foo', 'bar']),
        '')
    command = ['ifconfig', 'foo', 'bar']
    self.mock_logger.warning.assert_called_once_with(
        mock.ANY, command, b'error')

  @mock.patch('google_compute_engine.distro_lib.ip_forwarding_utils.subprocess')
  def testRunIfconfigException(self, mock_subprocess):
    mock_subprocess.Popen.side_effect = OSError('Test Error')

    self.assertEqual(
        self.mock_utils._RunIfconfig(args=['foo', 'bar']),
        '')
    command = ['ifconfig', 'foo', 'bar']
    self.mock_logger.warning.assert_called_once_with(
        mock.ANY, command, 'Test Error')

  @mock.patch('google_compute_engine.distro_lib.ip_forwarding_utils.netaddr')
  def testParseForwardedIps(self, mock_netaddr):
    self.assertEqual(self.mock_utils.ParseForwardedIps(None), [])
    self.assertEqual(self.mock_utils.ParseForwardedIps([]), [])
    self.assertEqual(self.mock_utils.ParseForwardedIps([None]), [])
    self.assertEqual(self.mock_utils.ParseForwardedIps(['invalid']), [])
    self.assertEqual(self.mock_utils.ParseForwardedIps(['1a1a1a1']), [])
    self.assertEqual(self.mock_utils.ParseForwardedIps(['1.1.1.1.1']), [])
    self.assertEqual(self.mock_utils.ParseForwardedIps(['1111.1.1.1']), [])
    self.assertEqual(self.mock_utils.ParseForwardedIps(['1.1.1.1111']), [])
    expected_calls = [
        mock.call.warning(mock.ANY, None),
        mock.call.warning(mock.ANY, 'invalid'),
        mock.call.warning(mock.ANY, '1a1a1a1'),
        mock.call.warning(mock.ANY, '1.1.1.1.1'),
        mock.call.warning(mock.ANY, '1111.1.1.1'),
        mock.call.warning(mock.ANY, '1.1.1.1111'),
    ]
    self.assertEqual(self.mock_logger.mock_calls, expected_calls)
    self.assertEqual(mock_netaddr.IPNetwork.mock_calls, [])

  @mock.patch('google_compute_engine.distro_lib.ip_forwarding_utils.netaddr')
  def testParseForwardedIpsComplex(self, mock_netaddr):
    def side_effect(arg):
      return [arg]
    mock_netaddr.IPNetwork.side_effect = side_effect
    forwarded_ips = {
        '{{}}\n\"hello\"\n!@#$%^&*()\n\n': False,
        '1111.1.1.1': False,
        '1.1.1.1': True,
        'hello': False,
        '123.123.123.123': True,
        '1.1.1.': False,
        '1.1.1.a': False,
        None: False,
        '1.0.0.0': True,
        '1.1.1.1/1': True,
        '1.1.1.1/11': True,
        '123.123.123.123/1': True,
        '123.123.123.123/123': False,
        '123.123.123.123/a': False,
        '123.123.123.123/': False,
    }
    input_ips = forwarded_ips.keys()
    valid_ips = [ip for ip, valid in forwarded_ips.items() if valid]
    invalid_ips = [ip for ip, valid in forwarded_ips.items() if not valid]

    self.assertEqual(self.mock_utils.ParseForwardedIps(input_ips), valid_ips)
    expected_calls = [mock.call.warning(mock.ANY, ip) for ip in invalid_ips]
    self.assertEqual(self.mock_logger.mock_calls, expected_calls)
    expected_calls = [mock.call.IPNetwork(ip) for ip in valid_ips]
    self.assertEqual(mock_netaddr.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.distro_lib.ip_forwarding_utils.netaddr')
  def testParseForwardedIpsSubnet(self, mock_netaddr):
    mock_netaddr.IPNetwork.return_value = ['1.1.1.1']
    forwarded_ips = [
        '1.1.1.1',
        '1.1.1.1/32',
        '1.1.1.1/1',
        '1.1.1.1/10',
        '1.1.1.1/24'
    ]
    for ip in forwarded_ips:
      self.assertEqual(self.mock_utils.ParseForwardedIps([ip]), ['1.1.1.1'])
    expected_calls = [mock.call.IPNetwork(ip) for ip in forwarded_ips]
    self.assertEqual(mock_netaddr.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.distro_lib.ip_forwarding_utils.netifaces')
  @mock.patch('google_compute_engine.distro_lib.ip_forwarding_utils.netaddr')
  def testGetForwardedIps(self, mock_netaddr, mock_netifaces):
    mock_netifaces.AF_INET = 0
    mock_netifaces.ifaddresses.return_value = [[
        {'addr': 'a', 'netmask': 'a mask'},
        {'addr': 'b', 'netmask': 'b mask'},
        {'addr': 'c', 'netmask': 'c mask'},
    ]]
    mock_netaddr.IPAddress().netmask_bits.return_value = 32
    mock_parse = mock.Mock()
    mock_parse.return_value = ['Test']
    self.mock_utils.ParseForwardedIps = mock_parse

    self.assertEqual(
        self.mock_utils.GetForwardedIps('interface', 'ip'), ['Test'])
    mock_netifaces.ifaddresses.assert_called_once_with('interface')
    mock_parse.assert_called_once_with(['a/32', 'b/32', 'c/32'])

  @mock.patch('google_compute_engine.distro_lib.ip_forwarding_utils.netifaces')
  def testGetForwardedIpsEmpty(self, mock_netifaces):
    mock_netifaces.AF_INET = 0
    mock_netifaces.ifaddresses.return_value = []

    self.assertEqual(
        self.mock_utils.GetForwardedIps('interface', 'ip'), [])

  @mock.patch('google_compute_engine.distro_lib.ip_forwarding_utils.netaddr')
  def testAddForwardedIp(self, mock_netaddr):
    mock_netaddr.IPNetwork.return_value = ['1.1.1.1']
    mock_run = mock.Mock()
    self.mock_utils._RunIfconfig = mock_run

    self.mock_utils.AddForwardedIp('1.1.1.1', 'interface')
    mock_netaddr.IPNetwork.assert_called_once_with('1.1.1.1')
    mock_run.assert_called_once_with(
        args=['interface', 'alias', '1.1.1.1/32'])

  @mock.patch('google_compute_engine.distro_lib.ip_forwarding_utils.netaddr')
  def testAddIpAlias(self, mock_netaddr):
    mock_netaddr.IPNetwork.return_value = [
       '1.1.1.0', '1.1.1.1', '1.1.1.2', '1.1.1.3'
    ]
    mock_run = mock.Mock()
    self.mock_utils._RunIfconfig = mock_run

    self.mock_utils.AddForwardedIp('1.1.1.1/30', 'interface')
    mock_netaddr.IPNetwork.assert_called_once_with('1.1.1.1/30')
    expected_calls = [
        mock.call(args=['interface', 'alias', '1.1.1.0/32']),
        mock.call(args=['interface', 'alias', '1.1.1.1/32']),
        mock.call(args=['interface', 'alias', '1.1.1.2/32']),
        mock.call(args=['interface', 'alias', '1.1.1.3/32'])
    ]
    self.assertEqual(mock_run.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.distro_lib.ip_forwarding_utils.netaddr')
  def testRemoveForwardedIp(self, mock_netaddr):
    mock_ip = mock.Mock()
    mock_ip.ip = '1.1.1.1'
    mock_netaddr.IPNetwork.return_value = mock_ip
    mock_run = mock.Mock()
    self.mock_utils._RunIfconfig = mock_run

    self.mock_utils.RemoveForwardedIp('1.1.1.1', 'interface')
    mock_netaddr.IPNetwork.assert_called_once_with('1.1.1.1')
    mock_run.assert_called_once_with(
        args=['interface', '-alias', '1.1.1.1'])

  @mock.patch('google_compute_engine.distro_lib.ip_forwarding_utils.netaddr')
  def testRemoveAliasIp(self, mock_netaddr):
    mock_ip = mock.Mock()
    mock_ip.ip = '1.1.1.1'
    mock_netaddr.IPNetwork.return_value = mock_ip
    mock_run = mock.Mock()
    self.mock_utils._RunIfconfig = mock_run

    self.mock_utils.RemoveForwardedIp('1.1.1.1/24', 'interface')
    mock_netaddr.IPNetwork.assert_called_once_with('1.1.1.1/24')
    mock_run.assert_called_once_with(
        args=['interface', '-alias', '1.1.1.1'])


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# Copyright 2017 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Unittest for compat.py module."""

import sys

import google_compute_engine.compat
from google_compute_engine.test_compat import mock
from google_compute_engine.test_compat import reload_import
from google_compute_engine.test_compat import unittest
from google_compute_engine.test_compat import urlretrieve


class CompatTest(unittest.TestCase):

  @mock.patch('google_compute_engine.compat.subprocess.check_call')
  def testCurlRetrieve(self, mock_call):
    url = 'http://www.example.com/script.sh'
    filename = None
    expected = ['curl', '--max-time', mock.ANY, '--retry', mock.ANY, '--', url]

    if sys.version_info < (2, 7, 9):
      urlretrieve.urlretrieve(url, filename)
      mock_call.assert_called_with(expected)
    else:
      pass

  @mock.patch('google_compute_engine.compat.subprocess.check_call')
  def testCurlRetrieveFilename(self, mock_call):
    url = 'http://www.example.com/script.sh'
    filename = '/tmp/filename.txt'
    expected = [
        'curl', '--max-time', mock.ANY, '--retry', mock.ANY, '-o', filename,
        '--', url,
    ]

    if sys.version_info < (2, 7, 9):
      urlretrieve.urlretrieve(url, filename)
      mock_call.assert_called_with(expected)
    else:
      pass

  @mock.patch('google_compute_engine.compat.subprocess.check_call')
  @mock.patch('google_compute_engine.compat.urlretrieve.urlretrieve')
  def testUrlRetrieve(self, mock_retrieve, mock_call):
    url = 'http://www.example.com/script.sh'
    filename = '/tmp/filename.txt'
    args = ['arg1', 'arg2', 'arg3']
    kwargs = {'kwarg1': 1, 'kwarg2': 2}

    if sys.version_info >= (2, 7, 9):
      urlretrieve.urlretrieve(url, filename, *args, **kwargs)
      mock_retrieve.assert_called_once_with(url, filename, *args, **kwargs)
      mock_call.assert_not_called()
    else:
      pass

  @mock.patch('google_compute_engine.compat.distro.linux_distribution')
  def testDistroCompatLinux(self, mock_call):
    test_cases = {
        ('Fedora', '28', ''):
            google_compute_engine.distro_lib.el_7.utils,
        ('debian', '8.10', ''):
            google_compute_engine.distro_lib.debian_8.utils,
        ('debian', '9.3', ''):
            google_compute_engine.distro_lib.debian_9.utils,
        ('debian', '10.3', ''):
            google_compute_engine.distro_lib.debian_9.utils,
        ('SUSE Linux Enterprise Server', '11', 'x86_64'):
            google_compute_engine.distro_lib.sles_11.utils,
        ('SUSE Linux Enterprise Server', '12', 'x86_64'):
            google_compute_engine.distro_lib.sles_12.utils,
        ('SUSE Linux Enterprise Server', '13', 'x86_64'):
            google_compute_engine.distro_lib.sles_12.utils,
        ('CentOS Linux', '6.4.3', 'Core'):
            google_compute_engine.distro_lib.el_6.utils,
        ('CentOS Linux', '7.4.1708', 'Core'):
            google_compute_engine.distro_lib.el_7.utils,
        ('CentOS Linux', '8.4.3', 'Core'):
            google_compute_engine.distro_lib.el_7.utils,
        ('Red Hat Enterprise Linux Server', '6.3.2', ''):
            google_compute_engine.distro_lib.el_6.utils,
        ('Red Hat Enterprise Linux Server', '7.4', ''):
            google_compute_engine.distro_lib.el_7.utils,
        ('Red Hat Enterprise Linux Server', '8.5.1', ''):
            google_compute_engine.distro_lib.el_7.utils,
        ('', '', ''):
            google_compute_engine.distro_lib.debian_9.utils,
        ('xxxx', 'xxxx', 'xxxx'):
            google_compute_engine.distro_lib.debian_9.utils,
    }

    for distro in test_cases:
      mock_call.return_value = distro
      reload_import(google_compute_engine.compat)
      self.assertEqual(
          test_cases[distro], google_compute_engine.compat.distro_utils)

  @mock.patch('google_compute_engine.compat.sys.platform', 'freebsd11')
  def testDistroCompatFreeBSD(self):
    reload_import(google_compute_engine.compat)
    self.assertEqual(
        google_compute_engine.distro_lib.freebsd_11.utils,
        google_compute_engine.compat.distro_utils)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# Copyright 2016 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Create a Python package of the Linux guest environment."""

import glob
import sys

import setuptools

install_requires = ['setuptools']
if sys.version_info < (3, 0):
  install_requires += ['boto']
if sys.version_info >= (3, 7):
  install_requires += ['distro']

setuptools.setup(
    author='Google Compute Engine Team',
    author_email='gc-team@google.com',
    description='Google Compute Engine',
    include_package_data=True,
    install_requires=install_requires,
    license='Apache Software License',
    long_description='Google Compute Engine guest environment.',
    name='google-compute-engine',
    packages=setuptools.find_packages(),
    url='https://github.com/GoogleCloudPlatform/compute-image-packages',
    version='20190801.00',
    # Entry points create scripts in /usr/bin that call a function.
    entry_points={
        'console_scripts': [
            'google_accounts_daemon=google_compute_engine.accounts.accounts_daemon:main',
            'google_clock_skew_daemon=google_compute_engine.clock_skew.clock_skew_daemon:main',
            'google_instance_setup=google_compute_engine.instance_setup.instance_setup:main',
            'google_network_daemon=google_compute_engine.networking.network_daemon:main',
            'google_metadata_script_runner=google_compute_engine.metadata_scripts.script_manager:main',
        ],
    },
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: No Input/Output (Daemon)',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Internet',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: System :: Installation/Setup',
        'Topic :: System :: Systems Administration',
    ],
)



#!/usr/bin/python
# Copyright 2016 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""A library for retrieving and modifying configuration settings."""

import os
import textwrap

from google_compute_engine import constants
from google_compute_engine import file_utils
from google_compute_engine.compat import parser

CONFIG = constants.SYSCONFDIR + '/instance_configs.cfg'


class ConfigManager(object):
  """Process the configuration defaults."""

  def __init__(self, config_file=None, config_header=None):
    """Constructor.

    Args:
      config_file: string, the location of the config file.
      config_header: string, the message to write at the top of the config.
    """
    self.config_file = config_file or CONFIG
    self.config_header = config_header
    self.config = parser.Parser()
    self.config.read(self.config_file)

  def _AddHeader(self, fp):
    """Create a file header in the config.

    Args:
      fp: int, a file pointer for writing the header.
    """
    text = textwrap.wrap(
        textwrap.dedent(self.config_header), break_on_hyphens=False)
    fp.write('\n'.join(['# ' + line for line in text]))
    fp.write('\n\n')

  def GetOptionString(self, section, option):
    """Get the value of an option in the config file.

    Args:
      section: string, the section of the config file to check.
      option: string, the option to retrieve the value of.

    Returns:
      string, the value of the option or None if the option doesn't exist.
    """
    if self.config.has_option(section, option):
      return self.config.get(section, option)
    else:
      return None

  def GetOptionBool(self, section, option):
    """Get the value of an option in the config file.

    Args:
      section: string, the section of the config file to check.
      option: string, the option to retrieve the value of.

    Returns:
      bool, True if the option is enabled or not set.
    """
    return (not self.config.has_option(section, option)
            or self.config.getboolean(section, option))

  def SetOption(self, section, option, value, overwrite=True):
    """Set the value of an option in the config file.

    Args:
      section: string, the section of the config file to check.
      option: string, the option to set the value of.
      value: string, the value to set the option.
      overwrite: bool, True to overwrite an existing value in the config file.
    """
    if not overwrite and self.config.has_option(section, option):
      return
    if not self.config.has_section(section):
      self.config.add_section(section)
    self.config.set(section, option, str(value))

  def WriteConfig(self, config_file=None):
    """Write the config values to a given file.

    Args:
      config_file: string, the file location of the config file to write.
    """
    config_file = config_file or self.config_file
    config_name = os.path.splitext(os.path.basename(config_file))[0]
    config_lock = (
        '%s/lock/google_%s.lock' % (constants.LOCALSTATEDIR, config_name))
    with file_utils.LockFile(config_lock):
      with open(config_file, 'w') as config_fp:
        if self.config_header:
          self._AddHeader(config_fp)
        self.config.write(config_fp)



#!/usr/bin/python
# Copyright 2016 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Run initialization code the first time the instance boots."""

import logging.handlers
import optparse
import os
import re
import shutil
import subprocess
import tempfile

from google_compute_engine import constants
from google_compute_engine import file_utils
from google_compute_engine import logger
from google_compute_engine import metadata_watcher
from google_compute_engine.boto import boto_config
from google_compute_engine.compat import urlerror
from google_compute_engine.compat import urlrequest
from google_compute_engine.instance_setup import instance_config


class PutRequest(urlrequest.Request):
  def get_method(self):
    return 'PUT'


GUEST_ATTRIBUTES_URL = ('http://metadata.google.internal/computeMetadata/v1/'
                        'instance/guest-attributes')
HOSTKEY_NAMESPACE = 'hostkeys'


class InstanceSetup(object):
  """Initialize the instance the first time it boots."""

  def __init__(self, debug=False):
    """Constructor.

    Args:
      debug: bool, True if debug output should write to the console.
    """
    self.debug = debug
    facility = logging.handlers.SysLogHandler.LOG_DAEMON
    self.logger = logger.Logger(
        name='instance-setup', debug=self.debug, facility=facility)
    self.watcher = metadata_watcher.MetadataWatcher(logger=self.logger)
    self.metadata_dict = None
    self.instance_config = instance_config.InstanceConfig(logger=self.logger)

    if self.instance_config.GetOptionBool('InstanceSetup', 'network_enabled'):
      self.metadata_dict = self.watcher.GetMetadata()
      instance_config_metadata = self._GetInstanceConfig()
      self.instance_config = instance_config.InstanceConfig(
          logger=self.logger, instance_config_metadata=instance_config_metadata)
      if self.instance_config.GetOptionBool('InstanceSetup', 'set_host_keys'):
        host_key_types = self.instance_config.GetOptionString(
            'InstanceSetup', 'host_key_types')
        self._SetSshHostKeys(host_key_types=host_key_types)
      if self.instance_config.GetOptionBool('InstanceSetup', 'set_boto_config'):
        self._SetupBotoConfig()
    if self.instance_config.GetOptionBool(
        'InstanceSetup', 'optimize_local_ssd'):
      self._RunScript('google_optimize_local_ssd')
    if self.instance_config.GetOptionBool('InstanceSetup', 'set_multiqueue'):
      self._RunScript('google_set_multiqueue')
    try:
      self.instance_config.WriteConfig()
    except (IOError, OSError) as e:
      self.logger.warning(str(e))

  def _GetInstanceConfig(self):
    """Get the instance configuration specified in metadata.

    Returns:
      string, the instance configuration data.
    """
    try:
      instance_data = self.metadata_dict['instance']['attributes']
    except KeyError:
      instance_data = {}
      self.logger.warning('Instance attributes were not found.')

    try:
      project_data = self.metadata_dict['project']['attributes']
    except KeyError:
      project_data = {}
      self.logger.warning('Project attributes were not found.')

    return (instance_data.get('google-instance-configs')
            or project_data.get('google-instance-configs'))

  def _RunScript(self, script):
    """Run a script and log the streamed script output.

    Args:
      script: string, the file location of an executable script.
    """
    process = subprocess.Popen(
        script, shell=True, stderr=subprocess.STDOUT, stdout=subprocess.PIPE)
    while True:
      for line in iter(process.stdout.readline, b''):
        self.logger.info(line.decode('utf-8').rstrip('\n'))
      if process.poll() is not None:
        break

  def _GetInstanceId(self):
    """Get the instance ID for this VM.

    Returns:
      string, the instance ID for the VM.
    """
    try:
      return str(self.metadata_dict['instance']['id'])
    except KeyError:
      self.logger.warning('Instance ID was not found in metadata.')
      return None

  def _GenerateSshKey(self, key_type, key_dest):
    """Generate a new SSH key.

    Args:
      key_type: string, the type of the SSH key.
      key_dest: string, a file location to store the SSH key.

    Returns:
      tuple, key_type and public key string.
    """
    # Create a temporary file to save the created RSA keys.
    with tempfile.NamedTemporaryFile(prefix=key_type, delete=True) as temp:
      temp_key = temp.name

    command = ['ssh-keygen', '-t', key_type, '-f', temp_key, '-N', '', '-q']
    try:
      self.logger.info('Generating SSH key %s.', key_dest)
      subprocess.check_call(command)
    except subprocess.CalledProcessError:
      self.logger.warning('Could not create SSH key %s.', key_dest)
      return

    shutil.move(temp_key, key_dest)
    shutil.move('%s.pub' % temp_key, '%s.pub' % key_dest)

    file_utils.SetPermissions(key_dest, mode=0o600)
    file_utils.SetPermissions('%s.pub' % key_dest, mode=0o644)
    with open('%s.pub' % key_dest, 'r') as pk:
     key_data = pk.read()

    key_values = key_data.split()
    if len(key_values) < 2:
      self.logger.warning('Could not read host key from %s.pub.', key_dest)
      return
    else:
      return key_values[0], key_values[1]

  def _WriteHostKeyToGuestAttributes(self, key_type, key_value):
    """Write a host key to guest attributes, ignoring errors."""
    headers = {'Metadata-Flavor': 'Google'}
    url = '%s/%s/%s' % (GUEST_ATTRIBUTES_URL, HOSTKEY_NAMESPACE, key_type)
    key_value = key_value.encode('utf-8')
    req = PutRequest(url, key_value, headers)
    try:
      response = urlrequest.urlopen(req)
      self.logger.debug(response)
      self.logger.info('Wrote %s host key to guest attributes.', key_type)
    except urlerror.HTTPError:
      self.logger.info('Unable to write %s host key to guest attributes.',
                       key_type)

  def _StartSshd(self):
    """Initialize the SSH daemon."""
    # Exit as early as possible.
    # Instance setup systemd scripts block sshd from starting.
    if os.path.exists(constants.LOCALBASE + '/bin/systemctl'):
      return
    elif (os.path.exists('/etc/init.d/ssh')
          or os.path.exists('/etc/init/ssh.conf')):
      subprocess.call(['service', 'ssh', 'start'])
      subprocess.call(['service', 'ssh', 'reload'])
    elif (os.path.exists('/etc/init.d/sshd')
          or os.path.exists('/etc/init/sshd.conf')):
      subprocess.call(['service', 'sshd', 'start'])
      subprocess.call(['service', 'sshd', 'reload'])

  def _SetSshHostKeys(self, host_key_types=None):
    """Regenerates SSH host keys when the VM is restarted with a new IP address.

    Booting a VM from an image with a known SSH key allows a number of attacks.
    This function will regenerating the host key whenever the IP address
    changes. This applies the first time the instance is booted, and each time
    the disk is used to boot a new instance.

    Args:
      host_key_types: string, a comma separated list of host key types.
    """
    section = 'Instance'
    instance_id = self._GetInstanceId()
    if instance_id != self.instance_config.GetOptionString(
        section, 'instance_id'):
      self.logger.info('Generating SSH host keys for instance %s.', instance_id)
      file_regex = re.compile(r'ssh_host_(?P<type>[a-z0-9]*)_key\Z')
      key_dir = '/etc/ssh'
      key_files = [f for f in os.listdir(key_dir) if file_regex.match(f)]
      key_types = host_key_types.split(',') if host_key_types else []
      key_types_files = ['ssh_host_%s_key' % key_type for key_type in key_types]
      for key_file in set(key_files) | set(key_types_files):
        key_type = file_regex.match(key_file).group('type')
        key_dest = os.path.join(key_dir, key_file)
        key_data = self._GenerateSshKey(key_type, key_dest)
        if key_data:
          self._WriteHostKeyToGuestAttributes(key_data[0], key_data[1])
      self._StartSshd()
      self.instance_config.SetOption(section, 'instance_id', str(instance_id))

  def _GetNumericProjectId(self):
    """Get the numeric project ID.

    Returns:
      string, the numeric project ID.
    """
    try:
      return str(self.metadata_dict['project']['numericProjectId'])
    except KeyError:
      self.logger.warning('Numeric project ID was not found in metadata.')
      return None

  def _SetupBotoConfig(self):
    """Set the boto config so GSUtil works with provisioned service accounts."""
    project_id = self._GetNumericProjectId()
    try:
      boto_config.BotoConfig(project_id, debug=self.debug)
    except (IOError, OSError) as e:
      self.logger.warning(str(e))


def main():
  parser = optparse.OptionParser()
  parser.add_option(
      '-d', '--debug', action='store_true', dest='debug',
      help='print debug output to the console.')
  (options, _) = parser.parse_args()
  InstanceSetup(debug=bool(options.debug))


if __name__ == '__main__':
  main()



#!/usr/bin/python
# Copyright 2016 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Unittest for instance_config.py module."""

from google_compute_engine.instance_setup import instance_config
from google_compute_engine.test_compat import mock
from google_compute_engine.test_compat import unittest


class InstanceConfigTest(unittest.TestCase):

  def setUp(self):
    self.mock_logger = mock.Mock()
    instance_config.InstanceConfig.instance_config = 'config'
    instance_config.InstanceConfig.instance_config_distro = 'distro'
    instance_config.InstanceConfig.instance_config_template = 'template'
    instance_config.InstanceConfig.instance_config_script = '/tmp/test.py'
    instance_config.InstanceConfig.instance_config_header = '%s %s'
    instance_config.InstanceConfig.instance_config_options = {
        'third': {
            'e': '3',
            'c': '1',
            'd': '2',
        },
        'first': {
            'a': 'false',
        },
        'second': {
            'b': 'true',
        },
    }

  @mock.patch('google_compute_engine.instance_setup.instance_config.os.path.exists')
  @mock.patch('google_compute_engine.instance_setup.instance_config.config_manager.ConfigManager.SetOption')
  @mock.patch('google_compute_engine.instance_setup.instance_config.config_manager.ConfigManager.__init__')
  def testInstanceConfig(self, mock_init, mock_set, mock_exists):
    mocks = mock.Mock()
    mocks.attach_mock(mock_init, 'init')
    mocks.attach_mock(mock_set, 'set')
    mocks.attach_mock(mock_exists, 'exists')
    mocks.attach_mock(self.mock_logger, 'logger')
    mock_exists.return_value = False

    instance_config.InstanceConfig(logger=self.mock_logger)
    expected_calls = [
        mock.call.init(
            config_file='template', config_header='/tmp/test.py template'),
        mock.call.exists('config'),
        mock.call.exists('distro'),
        mock.call.set('first', 'a', 'false', overwrite=False),
        mock.call.set('second', 'b', 'true', overwrite=False),
        mock.call.set('third', 'c', '1', overwrite=False),
        mock.call.set('third', 'd', '2', overwrite=False),
        mock.call.set('third', 'e', '3', overwrite=False),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.instance_setup.instance_config.os.path.exists')
  @mock.patch('google_compute_engine.instance_setup.instance_config.config_manager.ConfigManager.SetOption')
  @mock.patch('google_compute_engine.instance_setup.instance_config.config_manager.ConfigManager.__init__')
  def testInstanceConfigExists(self, mock_init, mock_set, mock_exists):
    config_parser = instance_config.parser.Parser()
    config_metadata = '[first]\na = true'
    mock_config = mock.create_autospec(instance_config.parser.Parser)
    with mock.patch(
        'google_compute_engine.instance_setup.instance_config'
        '.parser') as mock_parser:
      mock_config.read = mock.Mock()
      mock_config.sections = mock.Mock()
      mock_config.sections.return_value = ['a', 'b']
      mock_config.items = lambda key: {'key: %s' % key: 'value: %s' % key}
      mock_parser.Parser.side_effect = [
          config_parser, mock_config, mock_config]
      mocks = mock.Mock()
      mocks.attach_mock(mock_init, 'init')
      mocks.attach_mock(mock_set, 'set')
      mocks.attach_mock(mock_parser, 'parser')
      mocks.attach_mock(mock_config, 'config')
      mocks.attach_mock(mock_exists, 'exists')
      mocks.attach_mock(self.mock_logger, 'logger')
      mock_exists.return_value = True

      instance_config.InstanceConfig(
          logger=self.mock_logger, instance_config_metadata=config_metadata)
      expected_calls = [
          mock.call.init(
              config_file='template', config_header='/tmp/test.py template'),
          mock.call.parser.Parser(),
          mock.call.exists('config'),
          mock.call.parser.Parser(),
          mock.call.config.read('config'),
          mock.call.config.sections(),
          mock.call.exists('distro'),
          mock.call.parser.Parser(),
          mock.call.config.read('distro'),
          mock.call.config.sections(),
          mock.call.set('first', 'a', 'true', overwrite=False),
          mock.call.set('a', 'key: a', 'value: a', overwrite=False),
          mock.call.set('b', 'key: b', 'value: b', overwrite=False),
          mock.call.set('a', 'key: a', 'value: a', overwrite=False),
          mock.call.set('b', 'key: b', 'value: b', overwrite=False),
          mock.call.set('first', 'a', 'false', overwrite=False),
          mock.call.set('second', 'b', 'true', overwrite=False),
          mock.call.set('third', 'c', '1', overwrite=False),
          mock.call.set('third', 'd', '2', overwrite=False),
          mock.call.set('third', 'e', '3', overwrite=False)
      ]
      self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.instance_setup.instance_config.parser.Parser.read')
  @mock.patch('google_compute_engine.instance_setup.instance_config.os.path.exists')
  @mock.patch('google_compute_engine.instance_setup.instance_config.config_manager.ConfigManager.SetOption')
  @mock.patch('google_compute_engine.instance_setup.instance_config.config_manager.ConfigManager.__init__')
  def testInstanceConfigError(self, mock_init, mock_set, mock_exists, mock_read):
    mock_read.side_effect = instance_config.parser.ParsingError('Error')
    mocks = mock.Mock()
    mocks.attach_mock(mock_init, 'init')
    mocks.attach_mock(mock_set, 'set')
    mocks.attach_mock(mock_read, 'read')
    mocks.attach_mock(mock_exists, 'exists')
    mocks.attach_mock(self.mock_logger, 'logger')
    mock_exists.return_value = True

    instance_config.InstanceConfig(
        logger=self.mock_logger, instance_config_metadata='Error')
    expected_calls = [
        mock.call.init(
            config_file='template', config_header='/tmp/test.py template'),
        mock.call.logger.error(mock.ANY, mock.ANY),
        mock.call.exists('config'),
        mock.call.read('config'),
        mock.call.logger.error(mock.ANY, mock.ANY),
        mock.call.exists('distro'),
        mock.call.read('distro'),
        mock.call.logger.error(mock.ANY, mock.ANY),
        mock.call.set('first', 'a', 'false', overwrite=False),
        mock.call.set('second', 'b', 'true', overwrite=False),
        mock.call.set('third', 'c', '1', overwrite=False),
        mock.call.set('third', 'd', '2', overwrite=False),
        mock.call.set('third', 'e', '3', overwrite=False)
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.instance_setup.instance_config.config_manager.ConfigManager.WriteConfig')
  def testWriteConfig(self, mock_write):
    mock_config = instance_config.InstanceConfig()
    instance_config.InstanceConfig.WriteConfig(mock_config)
    mock_write.assert_called_once_with(config_file='config')


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# Copyright 2016 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Unittest for instance_setup.py module."""

import subprocess

from google_compute_engine.instance_setup import instance_setup
from google_compute_engine.test_compat import builtin
from google_compute_engine.test_compat import mock
from google_compute_engine.test_compat import unittest


class InstanceSetupTest(unittest.TestCase):

  def setUp(self):
    self.mock_instance_config = mock.Mock()
    self.mock_logger = mock.Mock()
    self.mock_setup = mock.create_autospec(instance_setup.InstanceSetup)
    self.mock_setup.debug = False
    self.mock_setup.instance_config = self.mock_instance_config
    self.mock_setup.logger = self.mock_logger

  @mock.patch('google_compute_engine.instance_setup.instance_setup.instance_config')
  @mock.patch('google_compute_engine.instance_setup.instance_setup.metadata_watcher')
  @mock.patch('google_compute_engine.instance_setup.instance_setup.logger')
  def testInstanceSetup(self, mock_logger, mock_watcher, mock_config):
    mock_setup = mock.create_autospec(instance_setup.InstanceSetup)
    mocks = mock.Mock()
    mocks.attach_mock(mock_logger, 'logger')
    mocks.attach_mock(mock_watcher, 'watcher')
    mocks.attach_mock(mock_config, 'config')
    mocks.attach_mock(mock_setup, 'setup')
    mock_logger_instance = mock.Mock()
    mock_logger.Logger.return_value = mock_logger_instance
    mock_watcher_instance = mock.Mock()
    mock_watcher_instance.GetMetadata.return_value = {'hello': 'world'}
    mock_watcher.MetadataWatcher.return_value = mock_watcher_instance
    mock_config_instance = mock.Mock()
    mock_config_instance.GetOptionBool.return_value = True
    mock_config_instance.GetOptionString.return_value = 'type'
    mock_config.InstanceConfig.return_value = mock_config_instance
    mock_setup._GetInstanceConfig.return_value = 'config'

    instance_setup.InstanceSetup.__init__(mock_setup)
    expected_calls = [
        # Setup and reading the configuration file.
        mock.call.logger.Logger(
            name=mock.ANY, debug=False, facility=mock.ANY),
        mock.call.watcher.MetadataWatcher(logger=mock_logger_instance),
        mock.call.config.InstanceConfig(logger=mock_logger_instance),
        # Check network access for reaching the metadata server.
        mock.call.config.InstanceConfig().GetOptionBool(
            'InstanceSetup', 'network_enabled'),
        mock.call.watcher.MetadataWatcher().GetMetadata(),
        # Get the instance config if specified in metadata.
        mock.call.setup._GetInstanceConfig(),
        mock.call.config.InstanceConfig(
            logger=mock_logger_instance, instance_config_metadata='config'),
        # Setup for SSH host keys if necessary.
        mock.call.config.InstanceConfig().GetOptionBool(
            'InstanceSetup', 'set_host_keys'),
        mock.call.config.InstanceConfig().GetOptionString(
            'InstanceSetup', 'host_key_types'),
        mock.call.setup._SetSshHostKeys(host_key_types='type'),
        # Setup for the boto config if necessary.
        mock.call.config.InstanceConfig().GetOptionBool(
            'InstanceSetup', 'set_boto_config'),
        mock.call.setup._SetupBotoConfig(),
        # Setup for local SSD.
        mock.call.config.InstanceConfig().GetOptionBool(
            'InstanceSetup', 'optimize_local_ssd'),
        mock.call.setup._RunScript('google_optimize_local_ssd'),
        # Setup for multiqueue virtio driver.
        mock.call.config.InstanceConfig().GetOptionBool(
            'InstanceSetup', 'set_multiqueue'),
        mock.call.setup._RunScript('google_set_multiqueue'),
        # Write the updated config file.
        mock.call.config.InstanceConfig().WriteConfig(),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)
    self.assertEqual(mock_setup.metadata_dict, {'hello': 'world'})

  @mock.patch('google_compute_engine.instance_setup.instance_setup.instance_config')
  @mock.patch('google_compute_engine.instance_setup.instance_setup.metadata_watcher')
  @mock.patch('google_compute_engine.instance_setup.instance_setup.logger')
  def testInstanceSetupException(self, mock_logger, mock_watcher, mock_config):
    mock_setup = mock.create_autospec(instance_setup.InstanceSetup)
    mocks = mock.Mock()
    mocks.attach_mock(mock_logger, 'logger')
    mocks.attach_mock(mock_watcher, 'watcher')
    mocks.attach_mock(mock_config, 'config')
    mocks.attach_mock(mock_setup, 'setup')
    mock_logger_instance = mock.Mock()
    mock_logger.Logger.return_value = mock_logger_instance
    mock_config_instance = mock.Mock()
    mock_config_instance.GetOptionBool.return_value = False
    mock_config_instance.WriteConfig.side_effect = IOError('Test Error')
    mock_config.InstanceConfig.return_value = mock_config_instance

    instance_setup.InstanceSetup.__init__(mock_setup)
    expected_calls = [
        mock.call.logger.Logger(
            name=mock.ANY, debug=False, facility=mock.ANY),
        mock.call.watcher.MetadataWatcher(logger=mock_logger_instance),
        mock.call.config.InstanceConfig(logger=mock_logger_instance),
        mock.call.config.InstanceConfig().GetOptionBool(
            'InstanceSetup', 'network_enabled'),
        mock.call.config.InstanceConfig().GetOptionBool(
            'InstanceSetup', 'optimize_local_ssd'),
        mock.call.config.InstanceConfig().GetOptionBool(
            'InstanceSetup', 'set_multiqueue'),
        mock.call.config.InstanceConfig().WriteConfig(),
        mock.call.logger.Logger().warning('Test Error'),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)
    self.assertIsNone(mock_setup.metadata_dict)

  def testGetInstanceConfig(self):
    instance_config = 'test'
    self.mock_setup.metadata_dict = {
        'instance': {
            'attributes': {
                'google-instance-configs': instance_config,
            }
        },
        'project': {
            'attributes': {
                'google-instance-configs': 'Unused config.',
            }
        }
    }
    self.assertEqual(
        instance_setup.InstanceSetup._GetInstanceConfig(self.mock_setup),
        instance_config)
    self.mock_logger.warning.assert_not_called()

  def testGetInstanceConfigProject(self):
    instance_config = 'test'
    self.mock_setup.metadata_dict = {
        'instance': {
            'attributes': {}
        },
        'project': {
            'attributes': {
                'google-instance-configs': instance_config,
            }
        }
    }
    self.assertEqual(
        instance_setup.InstanceSetup._GetInstanceConfig(self.mock_setup),
        instance_config)
    self.mock_logger.warning.assert_not_called()

  def testGetInstanceConfigNone(self):
    self.mock_setup.metadata_dict = {
        'instance': {
            'attributes': {}
        },
        'project': {
            'attributes': {}
        }
    }
    self.assertIsNone(
        instance_setup.InstanceSetup._GetInstanceConfig(self.mock_setup))
    self.mock_logger.warning.assert_not_called()

  def testGetInstanceConfigNoMetadata(self):
    self.mock_setup.metadata_dict = {}
    self.assertIsNone(
        instance_setup.InstanceSetup._GetInstanceConfig(self.mock_setup))
    self.assertEqual(self.mock_logger.warning.call_count, 2)

  @mock.patch('google_compute_engine.instance_setup.instance_setup.subprocess')
  def testRunScript(self, mock_subprocess):
    mock_readline = mock.Mock()
    mock_readline.side_effect = [bytes(b'a\n'), bytes(b'b\n'), bytes(b'')]
    mock_stdout = mock.Mock()
    mock_stdout.readline = mock_readline
    mock_process = mock.Mock()
    mock_process.poll.return_value = 0
    mock_process.stdout = mock_stdout
    mock_subprocess.Popen.return_value = mock_process
    script = '/tmp/script.py'

    instance_setup.InstanceSetup._RunScript(self.mock_setup, script)
    expected_calls = [mock.call('a'), mock.call('b')]
    self.assertEqual(self.mock_logger.info.mock_calls, expected_calls)
    mock_subprocess.Popen.assert_called_once_with(
        script, shell=True, stderr=mock_subprocess.STDOUT,
        stdout=mock_subprocess.PIPE)
    mock_process.poll.assert_called_once_with()

  def testGetInstanceId(self):
    self.mock_setup.metadata_dict = {'instance': {'attributes': {}, 'id': 123}}
    self.assertEqual(
        instance_setup.InstanceSetup._GetInstanceId(self.mock_setup), '123')
    self.mock_logger.warning.assert_not_called()

  def testGetInstanceIdNotFound(self):
    self.mock_setup.metadata_dict = {'instance': {'attributes': {}}}
    self.assertIsNone(
        instance_setup.InstanceSetup._GetInstanceId(self.mock_setup))
    self.assertEqual(self.mock_logger.warning.call_count, 1)

  @mock.patch('google_compute_engine.instance_setup.instance_setup.file_utils.SetPermissions')
  @mock.patch('google_compute_engine.instance_setup.instance_setup.shutil.move')
  @mock.patch('google_compute_engine.instance_setup.instance_setup.subprocess.check_call')
  @mock.patch('google_compute_engine.instance_setup.instance_setup.tempfile.NamedTemporaryFile')
  def testGenerateSshKey(
      self, mock_tempfile, mock_call, mock_move, mock_permissions):
    mocks = mock.Mock()
    mocks.attach_mock(mock_tempfile, 'tempfile')
    mocks.attach_mock(mock_call, 'call')
    mocks.attach_mock(mock_move, 'move')
    mocks.attach_mock(mock_permissions, 'permissions')
    mocks.attach_mock(self.mock_logger, 'logger')
    key_type = 'key-type'
    key_dest = '/key/dest'
    temp_dest = '/tmp/dest'
    mock_tempfile.return_value = mock_tempfile
    mock_tempfile.__enter__.return_value.name = temp_dest
    mock_open = mock.mock_open()
    key_file_contents = 'ssh-rsa asdfasdf'
    expected_key_data = ('ssh-rsa', 'asdfasdf')

    with mock.patch('%s.open' % builtin, mock_open, create=False):
      mock_open().read.return_value = key_file_contents
      key_data = instance_setup.InstanceSetup._GenerateSshKey(
          self.mock_setup, key_type, key_dest)
      expected_calls = [
          mock.call.tempfile(prefix=key_type, delete=True),
          mock.call.tempfile.__enter__(),
          mock.call.tempfile.__exit__(None, None, None),
          mock.call.logger.info(mock.ANY, key_dest),
          mock.call.call(
              ['ssh-keygen', '-t', key_type, '-f', temp_dest, '-N', '', '-q']),
          mock.call.move(temp_dest, key_dest),
          mock.call.move('%s.pub' % temp_dest, '%s.pub' % key_dest),
          mock.call.permissions(key_dest, mode=0o600),
          mock.call.permissions('%s.pub' % key_dest, mode=0o644),
      ]
      self.assertEqual(mocks.mock_calls, expected_calls)
      self.assertEqual(key_data, expected_key_data)

      mock_open().read.return_value = ''
      key_data = instance_setup.InstanceSetup._GenerateSshKey(
          self.mock_setup, key_type, key_dest)
      self.assertEqual(key_data, None)

  @mock.patch('google_compute_engine.instance_setup.instance_setup.subprocess.check_call')
  def testGenerateSshKeyProcessError(self, mock_call):
    key_type = 'key-type'
    key_dest = '/key/dest'
    mock_call.side_effect = subprocess.CalledProcessError(1, 'Test')

    instance_setup.InstanceSetup._GenerateSshKey(
        self.mock_setup, key_type, key_dest)
    self.mock_logger.info.assert_called_once_with(mock.ANY, key_dest)
    self.mock_logger.warning.assert_called_once_with(mock.ANY, key_dest)

  @mock.patch('google_compute_engine.instance_setup.instance_setup.subprocess.call')
  @mock.patch('google_compute_engine.instance_setup.instance_setup.os.path.exists')
  def testStartSshdSysVinit(self, mock_exists, mock_call):
    mocks = mock.Mock()
    mocks.attach_mock(mock_exists, 'exists')
    mocks.attach_mock(mock_call, 'call')
    mock_exists.side_effect = [False, False, True]

    instance_setup.InstanceSetup._StartSshd(self.mock_setup)
    expected_calls = [
        mock.call.exists('/bin/systemctl'),
        mock.call.exists('/etc/init.d/ssh'),
        mock.call.exists('/etc/init/ssh.conf'),
        mock.call.call(['service', 'ssh', 'start']),
        mock.call.call(['service', 'ssh', 'reload']),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.instance_setup.instance_setup.subprocess.call')
  @mock.patch('google_compute_engine.instance_setup.instance_setup.os.path.exists')
  def testStartSshdUpstart(self, mock_exists, mock_call):
    mocks = mock.Mock()
    mocks.attach_mock(mock_exists, 'exists')
    mocks.attach_mock(mock_call, 'call')
    mock_exists.side_effect = [False, False, False, False, True]

    instance_setup.InstanceSetup._StartSshd(self.mock_setup)
    expected_calls = [
        mock.call.exists('/bin/systemctl'),
        mock.call.exists('/etc/init.d/ssh'),
        mock.call.exists('/etc/init/ssh.conf'),
        mock.call.exists('/etc/init.d/sshd'),
        mock.call.exists('/etc/init/sshd.conf'),
        mock.call.call(['service', 'sshd', 'start']),
        mock.call.call(['service', 'sshd', 'reload']),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.instance_setup.instance_setup.subprocess.call')
  @mock.patch('google_compute_engine.instance_setup.instance_setup.os.path.exists')
  def testStartSshdSystemd(self, mock_exists, mock_call):
    mocks = mock.Mock()
    mocks.attach_mock(mock_exists, 'exists')
    mocks.attach_mock(mock_call, 'call')
    mock_exists.return_value = True

    instance_setup.InstanceSetup._StartSshd(self.mock_setup)
    expected_calls = [mock.call.exists('/bin/systemctl')]
    self.assertEqual(mocks.mock_calls, expected_calls)

  def testSetSshHostKeys(self):
    self.mock_instance_config.GetOptionString.return_value = '123'
    mock_instance_id = mock.Mock()
    mock_instance_id.return_value = '123'
    self.mock_setup._GetInstanceId = mock_instance_id

    instance_setup.InstanceSetup._SetSshHostKeys(self.mock_setup)
    self.mock_instance_config.SetOption.assert_not_called()

  @mock.patch('google_compute_engine.instance_setup.instance_setup.urlrequest.urlopen')
  @mock.patch('google_compute_engine.instance_setup.instance_setup.PutRequest')
  def testWriteHostKeyToGuestAttributes(self, mock_put, mock_urlopen):
    key_type = 'ssh-rsa'
    key_value = 'asdfasdf'
    encoded_key_value = key_value.encode('utf-8')
    expected_url = ('http://metadata.google.internal/computeMetadata/v1/'
                    'instance/guest-attributes/hostkeys/%s' % key_type)
    headers = {'Metadata-Flavor': 'Google'}

    instance_setup.InstanceSetup._WriteHostKeyToGuestAttributes(
        self.mock_setup, key_type, key_value)
    self.mock_logger.info.assert_called_with(
        'Wrote %s host key to guest attributes.', key_type)
    mock_put.assert_called_with(expected_url, encoded_key_value, headers)

    mock_urlopen.side_effect = instance_setup.urlerror.HTTPError(
        'http://foo', 403, 'Forbidden', {}, None)
    instance_setup.InstanceSetup._WriteHostKeyToGuestAttributes(
        self.mock_setup, key_type, key_value)
    self.mock_logger.info.assert_called_with(
        'Unable to write %s host key to guest attributes.', key_type)

  def testPutRequest(self):
    put_request = instance_setup.PutRequest('http://example.com/')
    self.assertEqual(put_request.get_method(), 'PUT')

  @mock.patch('google_compute_engine.instance_setup.instance_setup.os.listdir')
  def testSetSshHostKeysFirstBoot(self, mock_listdir):
    self.mock_instance_config.GetOptionString.return_value = None
    mock_instance_id = mock.Mock()
    mock_instance_id.return_value = '123'
    self.mock_setup._GetInstanceId = mock_instance_id
    mock_generate_key = mock.Mock()
    mock_generate_key.return_value = ('ssh-rsa', 'asdfasdf')
    self.mock_setup._GenerateSshKey = mock_generate_key
    mock_listdir.return_value = [
        'ssh_config',
        'ssh_host_dsa_key',
        'ssh_host_dsa_key.pub',
        'ssh_host_ed25519_key',
        'ssh_host_ed25519_key.pub',
        'ssh_host_rsa_key',
        'ssh_host_rsa_key.pub',
    ]

    instance_setup.InstanceSetup._SetSshHostKeys(
        self.mock_setup, host_key_types='rsa,dsa,abc')
    expected_calls = [
        mock.call('abc', '/etc/ssh/ssh_host_abc_key'),
        mock.call('dsa', '/etc/ssh/ssh_host_dsa_key'),
        mock.call('ed25519', '/etc/ssh/ssh_host_ed25519_key'),
        mock.call('rsa', '/etc/ssh/ssh_host_rsa_key'),
    ]

    self.assertEqual(sorted(mock_generate_key.mock_calls), expected_calls)
    self.mock_instance_config.SetOption.assert_called_once_with(
        'Instance', 'instance_id', '123')

  def testGetNumericProjectId(self):
    self.mock_setup.metadata_dict = {
        'project': {
            'attributes': {},
            'numericProjectId': 123,
        }
    }
    self.assertEqual(
        instance_setup.InstanceSetup._GetNumericProjectId(self.mock_setup),
        '123')
    self.mock_logger.warning.assert_not_called()

  def testGetNumericProjectIdNotFound(self):
    self.mock_setup.metadata_dict = {'project': {'attributes': {}}}
    self.assertIsNone(
        instance_setup.InstanceSetup._GetNumericProjectId(self.mock_setup))
    self.assertEqual(self.mock_logger.warning.call_count, 1)

  @mock.patch('google_compute_engine.instance_setup.instance_setup.boto_config.BotoConfig')
  def testSetupBotoConfig(self, mock_boto):
    mock_project_id = mock.Mock()
    mock_project_id.return_value = '123'
    self.mock_setup._GetNumericProjectId = mock_project_id
    instance_setup.InstanceSetup._SetupBotoConfig(self.mock_setup)
    mock_boto.assert_called_once_with('123', debug=False)

  @mock.patch('google_compute_engine.instance_setup.instance_setup.boto_config.BotoConfig')
  def testSetupBotoConfigLocked(self, mock_boto):
    mock_boto.side_effect = IOError('Test Error')
    instance_setup.InstanceSetup._SetupBotoConfig(self.mock_setup)
    self.mock_logger.warning.assert_called_once_with('Test Error')


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# Copyright 2016 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""A library for logging text to SysLog and the serial console."""

from google_compute_engine import constants
from google_compute_engine.compat import logging


def Logger(name, debug=False, facility=None):
  """Get a logging object with handlers for sending logs to SysLog.

  Args:
    name: string, the name of the logger which will be added to log entries.
    debug: bool, True if debug output should write to the console.
    facility: int, an encoding of the SysLog handler's facility and priority.

  Returns:
    logging object, an object for logging entries.
  """
  logger = logging.getLogger(name)
  logger.handlers = []
  logger.addHandler(logging.NullHandler())
  logger.propagate = False
  logger.setLevel(logging.DEBUG)
  formatter = logging.Formatter(name + ': %(levelname)s %(message)s')

  if debug:
    # Create a handler for console logging.
    console_handler = logging.StreamHandler()
    console_handler.setLevel(logging.DEBUG)
    console_handler.setFormatter(formatter)
    logger.addHandler(console_handler)

  if facility:
    # Create a handler for sending logs to SysLog.
    syslog_handler = logging.handlers.SysLogHandler(
        address=constants.SYSLOG_SOCKET, facility=facility)
    syslog_handler.setLevel(logging.INFO)
    syslog_handler.setFormatter(formatter)
    logger.addHandler(syslog_handler)

  return logger



#!/usr/bin/python
# Copyright 2016 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Utilities for configuring IP address forwarding."""

import logging
import os
import re
try:
  import netifaces
except ImportError:
  netifaces = None


MAC_REGEX = re.compile(r'\A([0-9A-Fa-f]{2}[:]){5}([0-9A-Fa-f]{2})\Z')


class NetworkUtils(object):
  """System network Ethernet interface utilities."""

  def __init__(self, logger=logging):
    """Constructor.

    Args:
      logger: logger object, used to write to SysLog and serial port.
    """
    self.logger = logger
    self.interfaces = self._CreateInterfaceMap()

  def _CreateInterfaceMap(self):
    """Generate a dictionary mapping MAC address to Ethernet interfaces.

    Returns:
      dict, string MAC addresses mapped to the string network interface name.
    """
    if netifaces:
      return self._CreateInterfaceMapNetifaces()
    else:
      return self._CreateInterfaceMapSysfs()

  def _CreateInterfaceMapSysfs(self):
    """Generate a dictionary mapping MAC address to Ethernet interfaces.

    Returns:
      dict, string MAC addresses mapped to the string network interface name.
    """
    interfaces = {}
    for interface in os.listdir('/sys/class/net'):
      try:
        mac_address = open(
            '/sys/class/net/%s/address' % interface).read().strip()
      except (IOError, OSError) as e:
        message = 'Unable to determine MAC address for %s. %s.'
        self.logger.warning(message, interface, str(e))
      else:
        interfaces[mac_address] = interface
    return interfaces

  def _CreateInterfaceMapNetifaces(self):
    """Generate a dictionary mapping MAC address to Ethernet interfaces.

    Returns:
      dict, string MAC addresses mapped to the string network interface name.
    """

    interfaces = {}
    for interface in netifaces.interfaces():
      af_link = netifaces.ifaddresses(interface).get(netifaces.AF_LINK, [])
      mac_address = next(iter(af_link), {}).get('addr', '')
      # In some systems this field can come with an empty string or with the
      # name of the interface when there is no MAC address associated with it.
      # Check the regex to be sure.
      if MAC_REGEX.match(mac_address):
        interfaces[mac_address] = interface
      else:
        message = 'Unable to determine MAC address for %s.'
        self.logger.warning(message, interface)
    return interfaces

  def GetNetworkInterface(self, mac_address):
    """Get the name of the network interface associated with a MAC address.

    Args:
      mac_address: string, the hardware address of the network interface.

    Returns:
      string, the network interface associated with a MAC address or None.
    """
    return self.interfaces.get(mac_address)



#!/usr/bin/python
# Copyright 2016 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Unittest for clock_skew_daemon.py module."""

from google_compute_engine.clock_skew import clock_skew_daemon
from google_compute_engine.test_compat import mock
from google_compute_engine.test_compat import unittest


class ClockSkewDaemonTest(unittest.TestCase):

  @mock.patch('google_compute_engine.clock_skew.clock_skew_daemon.metadata_watcher')
  @mock.patch('google_compute_engine.clock_skew.clock_skew_daemon.logger.Logger')
  @mock.patch('google_compute_engine.clock_skew.clock_skew_daemon.file_utils.LockFile')
  def testClockSkewDaemon(self, mock_lock, mock_logger, mock_watcher):
    mocks = mock.Mock()
    mocks.attach_mock(mock_lock, 'lock')
    mocks.attach_mock(mock_logger, 'logger')
    mocks.attach_mock(mock_watcher, 'watcher')
    metadata_key = clock_skew_daemon.ClockSkewDaemon.drift_token
    mock_logger.return_value = mock_logger
    mock_watcher.MetadataWatcher.return_value = mock_watcher
    with mock.patch.object(
        clock_skew_daemon.ClockSkewDaemon, 'HandleClockSync') as mock_handle:
      clock_skew_daemon.ClockSkewDaemon()
      expected_calls = [
          mock.call.logger(name=mock.ANY, debug=False, facility=mock.ANY),
          mock.call.watcher.MetadataWatcher(logger=mock_logger),
          mock.call.lock(clock_skew_daemon.LOCKFILE),
          mock.call.lock().__enter__(),
          mock.call.logger.info(mock.ANY),
          mock.call.watcher.WatchMetadata(
              mock_handle, metadata_key=metadata_key, recursive=False),
          mock.call.lock().__exit__(None, None, None),
      ]
      self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.clock_skew.clock_skew_daemon.metadata_watcher')
  @mock.patch('google_compute_engine.clock_skew.clock_skew_daemon.logger.Logger')
  @mock.patch('google_compute_engine.clock_skew.clock_skew_daemon.file_utils.LockFile')
  def testClockSkewDaemonError(self, mock_lock, mock_logger, mock_watcher):
    mocks = mock.Mock()
    mocks.attach_mock(mock_lock, 'lock')
    mocks.attach_mock(mock_logger, 'logger')
    mocks.attach_mock(mock_watcher, 'watcher')
    mock_lock.side_effect = IOError('Test Error')
    mock_logger.return_value = mock_logger
    with mock.patch.object(
        clock_skew_daemon.ClockSkewDaemon, 'HandleClockSync'):
      clock_skew_daemon.ClockSkewDaemon(debug=True)
      expected_calls = [
          mock.call.logger(name=mock.ANY, debug=True, facility=mock.ANY),
          mock.call.watcher.MetadataWatcher(logger=mock_logger),
          mock.call.lock(clock_skew_daemon.LOCKFILE),
          mock.call.logger.warning('Test Error'),
      ]
      self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.clock_skew.clock_skew_daemon.distro_utils')
  def testHandleClockSync(self, mock_distro_utils):
    mock_sync = mock.create_autospec(clock_skew_daemon.ClockSkewDaemon)
    mock_logger = mock.Mock()
    mock_sync.logger = mock_logger
    mock_sync.distro_utils = mock_distro_utils

    clock_skew_daemon.ClockSkewDaemon.HandleClockSync(mock_sync, 'Response')
    expected_calls = [mock.call.info(mock.ANY, 'Response')]
    self.assertEqual(mock_logger.mock_calls, expected_calls)
    expected_calls = [mock.call.HandleClockSync(mock_logger)]
    self.assertEqual(mock_distro_utils.mock_calls, expected_calls)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# Copyright 2016 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""A library used to set up the instance's default configurations file.

Note that the configurations in
/etc/default/instance_configs.cfg.template override the values set in
/etc/default/instance_configs.cfg. The system instance_configs.cfg may be
overridden during package upgrade.
"""

import logging
import os

from google_compute_engine import config_manager
from google_compute_engine import constants
from google_compute_engine.compat import parser
from google_compute_engine.compat import stringio


class InstanceConfig(config_manager.ConfigManager):
  """Creates a defaults config file for instance configuration."""

  instance_config = constants.SYSCONFDIR + '/instance_configs.cfg'
  instance_config_distro = '%s.distro' % instance_config
  instance_config_template = '%s.template' % instance_config
  instance_config_script = os.path.abspath(__file__)
  instance_config_header = (
      'This file is automatically created at boot time by the %s script. Do '
      'not edit this file directly. If you need to add items to this file, '
      'create or edit %s instead and then run '
      '/usr/bin/google_instance_setup.')
  instance_config_options = {
      'Accounts': {
          'deprovision_remove': 'false',
          'groups': 'adm,dip,docker,lxd,plugdev,video',

          # The encrypted password is set to '*' for SSH on Linux systems
          # without PAM.
          #
          # SSH uses '!' as its locked account token:
          # https://github.com/openssh/openssh-portable/blob/master/configure.ac
          #
          # When the token is specified, SSH denies login:
          # https://github.com/openssh/openssh-portable/blob/master/auth.c
          #
          # To solve the issue, make the password '*' which is also recognized
          # as locked but does not prevent SSH login.
          'gpasswd_add_cmd': 'gpasswd -a {user} {group}',
          'gpasswd_remove_cmd': 'gpasswd -d {user} {group}',
          'groupadd_cmd': 'groupadd {group}',
          'useradd_cmd': 'useradd -m -s /bin/bash -p * {user}',
          'userdel_cmd': 'userdel -r {user}',
          'usermod_cmd': 'usermod -G {groups} {user}',
      },
      'Daemons': {
          'accounts_daemon': 'true',
          'clock_skew_daemon': 'true',
          'ip_forwarding_daemon': 'true',  # Deprecated.
          'network_daemon': 'true',
      },
      'Instance': {
          'instance_id': '0',
      },
      'InstanceSetup': {
          'host_key_types': 'ecdsa,ed25519,rsa',
          'optimize_local_ssd': 'true',
          'network_enabled': 'true',
          # WARNING: Do not change the value of 'set_boto_config' without first
          # consulting the gsutil team (GoogleCloudPlatform/gsutil).
          'set_boto_config': 'true',
          'set_host_keys': 'true',
          'set_multiqueue': 'true',
      },
      'IpForwarding': {
          'ethernet_proto_id': '66',
          'ip_aliases': 'true',
          'target_instance_ips': 'true',
      },
      'MetadataScripts': {
          'run_dir': '',
          'startup': 'true',
          'shutdown': 'true',
          'default_shell': '/bin/bash',
      },
      'NetworkInterfaces': {
          'setup': 'true',
          'ip_forwarding': 'true',
          'dhcp_command': '',
          'dhclient_script': '/sbin/google-dhclient-script',
      },
  }

  def __init__(self, logger=logging, instance_config_metadata=None):
    """Constructor.

    Inherit from the ConfigManager class. Read the template for instance
    defaults and write new sections and options. This prevents package
    updates from overriding user set defaults.

    Args:
      logger: logger object, used to write to SysLog and serial port.
      instance_config_metadata: string, a config file specified in metadata.
    """
    self.logger = logger
    self.instance_config_metadata = instance_config_metadata
    self.instance_config_header %= (
        self.instance_config_script, self.instance_config_template)
    # User provided instance configs should always take precedence.
    super(InstanceConfig, self).__init__(
        config_file=self.instance_config_template,
        config_header=self.instance_config_header)

    # Use the instance config settings from metadata if specified. Then use
    # settings in an instance config file if one exists. If a config
    # file does not already exist, try to use the distro provided defaults. If
    # no file exists, use the default configuration settings.
    config_files = [self.instance_config, self.instance_config_distro]
    config_defaults = []
    if self.instance_config_metadata:
      config = parser.Parser()
      try:
        config.read_file(stringio.StringIO(self.instance_config_metadata))
      except parser.Error as e:
        self.logger.error('Error parsing metadata configs: %s', str(e))
      else:
        config_defaults.append(
            dict((s, dict(config.items(s))) for s in config.sections()))
    for config_file in config_files:
      if os.path.exists(config_file):
        config = parser.Parser()
        try:
          config.read(config_file)
        except parser.Error as e:
          self.logger.error('Error parsing config file: %s', str(e))
        else:
          config_defaults.append(
              dict((s, dict(config.items(s))) for s in config.sections()))
    config_defaults.append(self.instance_config_options)

    for defaults in config_defaults:
      for section, options in sorted(defaults.items()):
        for option, value in sorted(options.items()):
          super(InstanceConfig, self).SetOption(
              section, option, value, overwrite=False)

  def WriteConfig(self):
    """Write the config values to the instance defaults file."""
    super(InstanceConfig, self).WriteConfig(config_file=self.instance_config)



#!/usr/bin/python
# Copyright 2016 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Unittest for network_setup.py module."""

import subprocess

from google_compute_engine.networking.network_setup import network_setup
from google_compute_engine.test_compat import mock
from google_compute_engine.test_compat import unittest


class NetworkSetupTest(unittest.TestCase):

  def setUp(self):
    self.mock_logger = mock.Mock()
    self.mock_distro_utils = mock.Mock()
    self.dhclient_script = '/bin/script'
    self.dhcp_command = ''
    self.setup = network_setup.NetworkSetup(
        dhclient_script=self.dhclient_script, dhcp_command=self.dhcp_command,
        debug=False)
    self.setup.distro_utils = self.mock_distro_utils
    self.setup.logger = self.mock_logger

  @mock.patch('google_compute_engine.networking.network_setup.network_setup.subprocess.check_call')
  def testEnableIpv6(self, mock_call):
    mocks = mock.Mock()
    mocks.attach_mock(mock_call, 'call')
    mocks.attach_mock(self.mock_logger, 'logger')
    mocks.attach_mock(self.mock_distro_utils.EnableIpv6, 'enable')
    mock_call.side_effect = [None, subprocess.CalledProcessError(1, 'Test')]

    # Return immediately with no interfaces.
    network_setup.NetworkSetup.EnableIpv6(self.setup, None)
    network_setup.NetworkSetup.EnableIpv6(self.setup, [])
    # Enable interfaces.
    network_setup.NetworkSetup.EnableIpv6(self.setup, ['A', 'B'])
    self.assertEqual(self.setup.ipv6_interfaces, set(['A', 'B']))
    # Add a new interface.
    network_setup.NetworkSetup.EnableIpv6(self.setup, ['A', 'B', 'C'])
    self.assertEqual(self.setup.ipv6_interfaces, set(['A', 'B', 'C']))
    # Interfaces are already enabled, do nothing.
    network_setup.NetworkSetup.EnableIpv6(self.setup, ['A', 'B', 'C'])
    self.assertEqual(self.setup.ipv6_interfaces, set(['A', 'B', 'C']))
    expected_calls = [
        mock.call.logger.info(mock.ANY, ['A', 'B']),
        mock.call.enable(['A', 'B'], mock.ANY, dhclient_script='/bin/script'),
        mock.call.logger.info(mock.ANY, ['A', 'B', 'C']),
        mock.call.enable(
            ['A', 'B', 'C'], mock.ANY, dhclient_script='/bin/script'),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.networking.network_setup.network_setup.subprocess.check_call')
  def testDisableIpv6(self, mock_call):
    mocks = mock.Mock()
    mocks.attach_mock(mock_call, 'call')
    mocks.attach_mock(self.mock_logger, 'logger')
    mocks.attach_mock(self.mock_distro_utils.EnableIpv6, 'enable')
    mocks.attach_mock(self.mock_distro_utils.DisableIpv6, 'disable')
    expected_calls = []

    # Clean run, run disable once e.g. at boot.
    network_setup.NetworkSetup.DisableIpv6(self.setup, ['A'])
    self.assertEqual(self.setup.ipv6_interfaces, set([]))
    # No more disables allowed, have to follow the contract of Enable and then
    # Disable.
    network_setup.NetworkSetup.DisableIpv6(self.setup, ['A'])
    expected_calls.extend(
        [
            mock.call.logger.info(mock.ANY, ['A']),
            mock.call.disable(['A'], mock.ANY),
        ])
    # Enable interfaces.
    network_setup.NetworkSetup.EnableIpv6(self.setup, ['A', 'B', 'C'])
    expected_calls.extend(
        [
            mock.call.logger.info(mock.ANY, ['A', 'B', 'C']),
            mock.call.enable(
                ['A', 'B', 'C'], mock.ANY, dhclient_script='/bin/script'),
        ])
    # Remove interface.
    network_setup.NetworkSetup.DisableIpv6(self.setup, ['A'])
    self.assertEqual(self.setup.ipv6_interfaces, set(['B', 'C']))
    expected_calls.extend(
        [
            mock.call.logger.info(mock.ANY, ['A']),
            mock.call.disable(['A'], mock.ANY),
        ])

    # Add it back.
    network_setup.NetworkSetup.EnableIpv6(self.setup, ['A'])
    self.assertEqual(self.setup.ipv6_interfaces, set(['A', 'B', 'C']))
    expected_calls.extend(
        [
            mock.call.logger.info(mock.ANY, ['A']),
            mock.call.enable(['A'], mock.ANY, dhclient_script='/bin/script'),
        ])

    # Remove list.
    network_setup.NetworkSetup.DisableIpv6(self.setup, ['A', 'B'])
    self.assertEqual(self.setup.ipv6_interfaces, set(['C']))
    expected_calls.extend(
        [
            mock.call.logger.info(mock.ANY, ['A', 'B']),
            mock.call.disable(['A', 'B'], mock.ANY),
        ])

    # Try removing again, these are no ops.
    network_setup.NetworkSetup.DisableIpv6(self.setup, ['A'])
    network_setup.NetworkSetup.DisableIpv6(self.setup, ['A', 'B'])

    # Remove the last element.
    network_setup.NetworkSetup.DisableIpv6(self.setup, ['C'])
    self.assertEqual(self.setup.ipv6_interfaces, set([]))
    expected_calls.extend(
        [
            mock.call.logger.info(mock.ANY, ['C']),
            mock.call.disable(['C'], mock.ANY),
        ])

    # Empty list, allow adds back again.
    network_setup.NetworkSetup.EnableIpv6(self.setup, ['A'])
    self.assertEqual(self.setup.ipv6_interfaces, set(['A']))
    expected_calls.extend(
        [
            mock.call.logger.info(mock.ANY, ['A']),
            mock.call.enable(['A'], mock.ANY, dhclient_script='/bin/script'),
        ])
    self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.networking.network_setup.network_setup.subprocess.check_call')
  def testEnableNetworkInterfaces(self, mock_call):
    mocks = mock.Mock()
    mocks.attach_mock(mock_call, 'call')
    mocks.attach_mock(self.mock_logger, 'logger')
    mocks.attach_mock(self.mock_distro_utils.EnableNetworkInterfaces, 'enable')
    mock_call.side_effect = [None, subprocess.CalledProcessError(1, 'Test')]

    # Return immediately with no interfaces.
    network_setup.NetworkSetup.EnableNetworkInterfaces(self.setup, None)
    network_setup.NetworkSetup.EnableNetworkInterfaces(self.setup, [])
    # Enable interfaces.
    network_setup.NetworkSetup.EnableNetworkInterfaces(
        self.setup, ['A', 'B'])
    self.assertEqual(self.setup.interfaces, set(['A', 'B']))
    # Add a new interface.
    network_setup.NetworkSetup.EnableNetworkInterfaces(
        self.setup, ['A', 'B', 'C'])
    self.assertEqual(self.setup.interfaces, set(['A', 'B', 'C']))
    # Interfaces are already enabled.
    network_setup.NetworkSetup.EnableNetworkInterfaces(
        self.setup, ['A', 'B', 'C'])
    self.assertEqual(self.setup.interfaces, set(['A', 'B', 'C']))
    # Run a user supplied command successfully.
    self.setup.dhcp_command = 'success'
    network_setup.NetworkSetup.EnableNetworkInterfaces(
        self.setup, ['D', 'E'])
    self.assertEqual(self.setup.interfaces, set(['D', 'E']))
    # Run a user supplied command and logger error messages.
    self.setup.dhcp_command = 'failure'
    network_setup.NetworkSetup.EnableNetworkInterfaces(
        self.setup, ['F', 'G'])
    self.assertEqual(self.setup.interfaces, set(['F', 'G']))
    expected_calls = [
        mock.call.logger.info(mock.ANY, ['A', 'B']),
        mock.call.enable(['A', 'B'], mock.ANY, dhclient_script='/bin/script'),
        mock.call.logger.info(mock.ANY, ['A', 'B', 'C']),
        mock.call.enable(
            ['A', 'B', 'C'], mock.ANY, dhclient_script='/bin/script'),
        mock.call.logger.info(mock.ANY, ['D', 'E']),
        mock.call.call(['success']),
        mock.call.logger.info(mock.ANY, ['F', 'G']),
        mock.call.call(['failure']),
        mock.call.logger.warning(mock.ANY),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)



#!/usr/bin/python
# Copyright 2016 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Unittest for script_manager.py module."""

from google_compute_engine.metadata_scripts import script_manager
from google_compute_engine.test_compat import mock
from google_compute_engine.test_compat import unittest


class ScriptManagerTest(unittest.TestCase):

  @mock.patch('google_compute_engine.metadata_scripts.script_manager.script_retriever')
  @mock.patch('google_compute_engine.metadata_scripts.script_manager.logger')
  @mock.patch('google_compute_engine.metadata_scripts.script_manager.script_executor')
  @mock.patch('google_compute_engine.metadata_scripts.script_manager.shutil.rmtree')
  @mock.patch('google_compute_engine.metadata_scripts.script_manager.tempfile.mkdtemp')
  def testRunScripts(
      self, mock_mkdir, mock_rmtree, mock_executor, mock_logger,
      mock_retriever):
    mock_logger_instance = mock.Mock()
    mock_logger.Logger.return_value = mock_logger_instance
    mock_retriever_instance = mock.Mock()
    mock_retriever.ScriptRetriever.return_value = mock_retriever_instance
    mocks = mock.Mock()
    mocks.attach_mock(mock_mkdir, 'mkdir')
    mocks.attach_mock(mock_rmtree, 'rmtree')
    mocks.attach_mock(mock_executor, 'executor')
    mocks.attach_mock(mock_logger, 'logger')
    mocks.attach_mock(mock_retriever, 'retriever')
    run_dir = '/var/run'
    script_type = 'test'
    script_name = '%s-script' % script_type
    script_prefix = '%s-' % script_type
    test_dir = 'test-dir'
    test_dict = {'test': 'dict'}
    mock_mkdir.return_value = test_dir
    mock_retriever_instance.GetScripts.return_value = test_dict

    script_manager.ScriptManager(script_type, run_dir=run_dir)
    expected_calls = [
        mock.call.logger.Logger(
            name=script_name, debug=False, facility=mock.ANY),
        mock.call.retriever.ScriptRetriever(mock_logger_instance, script_type),
        mock.call.executor.ScriptExecutor(
            mock_logger_instance, script_type, default_shell=None),
        mock.call.mkdir(prefix=script_prefix, dir=run_dir),
        mock.call.logger.Logger().info(mock.ANY, script_type),
        mock.call.retriever.ScriptRetriever().GetScripts(test_dir),
        mock.call.executor.ScriptExecutor().RunScripts(test_dict),
        mock.call.logger.Logger().info(mock.ANY, script_type),
        mock.call.rmtree(test_dir),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# Copyright 2016 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Authentication module for using Google Compute service accounts."""

from boto import auth_handler
from google_compute_engine import logger
from google_compute_engine import metadata_watcher

GS_SCOPES = set([
    'https://www.googleapis.com/auth/devstorage.read_only',
    'https://www.googleapis.com/auth/devstorage.read_write',
    'https://www.googleapis.com/auth/devstorage.full_control',
])


class ComputeAuth(auth_handler.AuthHandler):
  """Google Compute service account auth handler.

  The boto library reads the system config file (/etc/boto.cfg) and looks
  at a config value called 'plugin_directory'. It then loads the Python
  files and find classes derived from boto.auth_handler.AuthHandler.
  """

  capability = ['google-oauth2', 's3']
  metadata_key = 'instance/service-accounts'

  def __init__(self, path, config, provider):
    self.logger = logger.Logger(name='compute-auth')
    self.watcher = metadata_watcher.MetadataWatcher(logger=self.logger)
    self.service_account = config.get('GoogleCompute', 'service_account', '')
    self.scopes = None
    if provider.name == 'google' and self.service_account:
      self.scopes = self._GetGsScopes()
    if not self.scopes:
      raise auth_handler.NotReadyToAuthenticate()

  def _GetGsScopes(self):
    """Return all Google Storage scopes available on this VM."""
    service_accounts = self.watcher.GetMetadata(metadata_key=self.metadata_key)
    try:
      scopes = service_accounts[self.service_account]['scopes']
      return list(GS_SCOPES.intersection(set(scopes))) if scopes else None
    except KeyError:
      return None

  def _GetAccessToken(self):
    """Return an OAuth 2.0 access token for Google Storage."""
    service_accounts = self.watcher.GetMetadata(metadata_key=self.metadata_key)
    try:
      return service_accounts[self.service_account]['token']['access_token']
    except KeyError:
      return None

  def add_auth(self, http_request):
    http_request.headers['Authorization'] = 'OAuth %s' % self._GetAccessToken()



#!/usr/bin/python
# Copyright 2016 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Unittest for logger.py module."""

from google_compute_engine import logger
from google_compute_engine.test_compat import mock
from google_compute_engine.test_compat import unittest


class LoggerTest(unittest.TestCase):

  @mock.patch('google_compute_engine.logger.logging.handlers.SysLogHandler')
  @mock.patch('google_compute_engine.logger.logging.StreamHandler')
  @mock.patch('google_compute_engine.logger.logging.NullHandler')
  def testLogger(self, mock_null, mock_stream, mock_syslog):
    mock_null.return_value = mock_null
    mock_stream.return_value = mock_stream
    mock_syslog.return_value = mock_syslog
    name = 'test'

    # Verify basic logger setup.
    named_logger = logger.Logger(name=name, debug=True)
    mock_stream.setLevel.assert_called_once_with(logger.logging.DEBUG)
    self.assertEqual(named_logger.handlers, [mock_null, mock_stream])

    # Verify logger setup with a facility.
    address = '/dev/log'
    facility = 1
    named_logger = logger.Logger(name=name, debug=True, facility=facility)
    mock_syslog.assert_called_once_with(address=address, facility=facility)
    mock_syslog.setLevel.assert_called_once_with(logger.logging.INFO)
    self.assertEqual(
        named_logger.handlers, [mock_null, mock_stream, mock_syslog])

    # Verify the handlers are reset during repeated calls.
    named_logger = logger.Logger(name=name, debug=False)
    self.assertEqual(named_logger.handlers, [mock_null])


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# Copyright 2016 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Unittest for network_utils.py module."""

from google_compute_engine import network_utils
from google_compute_engine.test_compat import builtin
from google_compute_engine.test_compat import mock
from google_compute_engine.test_compat import unittest


class NetworkUtilsTest(unittest.TestCase):

  def setUp(self):
    self.mock_logger = mock.Mock()
    self.interfaces = {'address': 'interface'}
    self.mock_utils = network_utils.NetworkUtils(self.mock_logger)
    self.mock_utils.interfaces = self.interfaces

  @mock.patch('google_compute_engine.network_utils.netifaces', False)
  @mock.patch('google_compute_engine.network_utils.os.listdir')
  def testCreateInterfaceMapSysfs(self, mock_listdir):
    mock_open = mock.mock_open()
    interface_map = {
        '1': 'a',
        '2': 'b',
        '3': 'c',
    }
    mock_listdir.return_value = interface_map.values()

    with mock.patch('%s.open' % builtin, mock_open, create=False):
      addresses = interface_map.keys()
      addresses = ['%s\n' % address for address in addresses]
      mock_open().read.side_effect = interface_map.keys()
      self.assertEqual(self.mock_utils._CreateInterfaceMap(), interface_map)

  @mock.patch('google_compute_engine.network_utils.netifaces', False)
  @mock.patch('google_compute_engine.network_utils.os.listdir')
  def testCreateInterfaceMapSysfsError(self, mock_listdir):
    mock_open = mock.mock_open()
    mock_listdir.return_value = ['a', 'b', 'c']

    with mock.patch('%s.open' % builtin, mock_open, create=False):
      mock_open().read.side_effect = [
          '1', OSError('OSError'), IOError('IOError')]
      self.assertEqual(self.mock_utils._CreateInterfaceMap(), {'1': 'a'})
      expected_calls = [
          mock.call.warning(mock.ANY, 'b', 'OSError'),
          mock.call.warning(mock.ANY, 'c', 'IOError'),
      ]
      self.assertEqual(self.mock_logger.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.network_utils.netifaces.AF_LINK', 88)
  @mock.patch('google_compute_engine.network_utils.netifaces')
  def testCreateInterfaceMapNetifaces(self, mock_netifaces):
    interface_map = {
        '11:11:11:11:11:11': 'a',
        '22:22:22:22:22:22': 'b',
        '33:33:33:33:33:33': 'c',
    }
    ifaddress_map = {
        'a': {mock_netifaces.AF_LINK: [{'addr': '11:11:11:11:11:11'}]},
        'b': {mock_netifaces.AF_LINK: [{'addr': '22:22:22:22:22:22'}]},
        'c': {mock_netifaces.AF_LINK: [{'addr': '33:33:33:33:33:33'}]},
    }
    mock_netifaces.interfaces.return_value = interface_map.values()
    mock_netifaces.ifaddresses.side_effect = (
        lambda interface: ifaddress_map[interface])
    self.assertEqual(self.mock_utils._CreateInterfaceMap(), interface_map)

  @mock.patch('google_compute_engine.network_utils.netifaces.AF_LINK', 88)
  @mock.patch('google_compute_engine.network_utils.netifaces')
  def testCreateInterfaceMapNetifacesError(self, mock_netifaces):
    ifaddress_map = {
        'a': {mock_netifaces.AF_LINK: [{'addr': '11:11:11:11:11:11'}]},
        'b': {},
        'c': {mock_netifaces.AF_LINK: [{'addr': ''}]},
    }
    mock_netifaces.interfaces.return_value = ['a', 'b', 'c']
    mock_netifaces.ifaddresses.side_effect = (
        lambda interface: ifaddress_map[interface])

    self.assertEqual(
        self.mock_utils._CreateInterfaceMap(), {'11:11:11:11:11:11': 'a'})
    expected_calls = [
        mock.call.warning(mock.ANY, 'b'),
        mock.call.warning(mock.ANY, 'c'),
    ]
    self.assertEqual(self.mock_logger.mock_calls, expected_calls)

  def testGetNetworkInterface(self):
    self.assertIsNone(self.mock_utils.GetNetworkInterface('invalid'))
    self.assertEqual(
        self.mock_utils.GetNetworkInterface('address'), 'interface')



#!/usr/bin/python
# Copyright 2016 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""A module for resolving compatibility issues between Python 2 and Python 3."""

import logging
import subprocess
import sys

if sys.version_info >= (3, 7):
  import distro
else:
  import platform as distro

if 'freebsd' in sys.platform:
  # Note: Do not use .version() method which is from either platform or distro.
  # platform.version() and distro.version() return different values.
  # platform.version() returns 'FreeBSD 11.2-RELEASE-p9.....'.
  # distro.version() returns '11.2'.
  distro_name = 'freebsd'
  # distro_version is not used for FreeBSD later in this code.
  distro_version = None
else:
  distribution = distro.linux_distribution()
  distro_name = distribution[0].lower()
  distro_version = distribution[1].split('.')[0]
distro_utils = None

if 'centos' in distro_name and distro_version == '6':
  import google_compute_engine.distro_lib.el_6.utils as distro_utils
elif 'centos' in distro_name:
  import google_compute_engine.distro_lib.el_7.utils as distro_utils
elif 'red hat enterprise linux' in distro_name and distro_version == '6':
  import google_compute_engine.distro_lib.el_6.utils as distro_utils
elif 'red hat enterprise linux' in distro_name:
  import google_compute_engine.distro_lib.el_7.utils as distro_utils
elif 'fedora' in distro_name:
  import google_compute_engine.distro_lib.el_7.utils as distro_utils
elif 'debian' in distro_name and distro_version == '8':
  import google_compute_engine.distro_lib.debian_8.utils as distro_utils
elif 'debian' in distro_name:
  import google_compute_engine.distro_lib.debian_9.utils as distro_utils
elif 'suse' in distro_name and distro_version == '11':
  import google_compute_engine.distro_lib.sles_11.utils as distro_utils
elif 'suse' in distro_name:
  import google_compute_engine.distro_lib.sles_12.utils as distro_utils
elif 'freebsd' in distro_name:
  import google_compute_engine.distro_lib.freebsd_11.utils as distro_utils
else:
  # Default to Debian 9.
  import google_compute_engine.distro_lib.debian_9.utils as distro_utils

RETRY_LIMIT = 3
TIMEOUT = 10

if sys.version_info >= (3, 0):
  # Python 3 imports.
  import configparser as parser
  import http.client as httpclient
  import io as stringio
  import urllib.error as urlerror
  import urllib.parse as urlparse
  import urllib.request as urlrequest
  import urllib.request as urlretrieve
else:
  # Python 2 imports.
  import ConfigParser as parser
  import httplib as httpclient
  import StringIO as stringio
  import urllib as urlparse
  import urllib as urlretrieve
  import urllib2 as urlrequest
  import urllib2 as urlerror

if sys.version_info < (2, 7):

  class NullHandler(logging.Handler):

    def emit(self, record):
      pass

    def handle(self, record):
      pass

    def createLock(self):
      pass

  logging.NullHandler = NullHandler

if sys.version_info < (2, 7, 9):

  # Native Python libraries do not check SSL certificates.
  def curlretrieve(url, filename=None, *args, **kwargs):
    command = ['curl', '--max-time', str(TIMEOUT), '--retry', str(RETRY_LIMIT)]
    if filename:
      command += ['-o', filename]
    command += ['--', url]
    subprocess.check_call(command)

  urlretrieve.urlretrieve = curlretrieve

if sys.version_info < (3, 2):
  parser.SafeConfigParser.read_file = parser.SafeConfigParser.readfp
  parser.Parser = parser.SafeConfigParser
else:
  parser.Parser = parser.ConfigParser



#!/usr/bin/python
# Copyright 2016 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""A module for resolving compatibility issues between Python 2 and Python 3."""

import sys

from google_compute_engine.compat import httpclient
from google_compute_engine.compat import parser
from google_compute_engine.compat import urlerror
from google_compute_engine.compat import urlparse
from google_compute_engine.compat import urlrequest
from google_compute_engine.compat import urlretrieve

# Import the mock module in Python 3.2.
if sys.version_info >= (3, 3):
  import unittest.mock as mock
else:
  import mock

# Import the unittest2 module to backport testing features to Python 2.6.
if sys.version_info >= (2, 7):
  import unittest
else:
  import unittest2 as unittest

builtin = 'builtins' if sys.version_info >= (3,) else '__builtin__'

# Import the reload module to re-import modules for testing compat.
if sys.version_info >= (3, 4):
  from importlib import reload as reload_import
else:
  from imp import reload as reload_import



#!/usr/bin/python
# Copyright 2018 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Utilities that are distro specific for use on SUSE 12."""

import os
import subprocess

from google_compute_engine import constants
from google_compute_engine.distro_lib import helpers
from google_compute_engine.distro_lib import ip_forwarding_utils
from google_compute_engine.distro_lib import utils


class Utils(utils.Utils):
  """Utilities used by Linux guest services on SUSE 12."""

  network_path = constants.LOCALBASE + '/etc/sysconfig/network'

  def EnableIpv6(self, interfaces, logger, dhclient_script=None):
    """Configure the network interfaces for IPv6 using dhclient.

    Args:
      interface: string, the output device names for enabling IPv6.
      logger: logger object, used to write to SysLog and serial port.
      dhclient_script: string, the path to a dhclient script used by dhclient.
    """
    pass

  def EnableNetworkInterfaces(self, interfaces, logger, dhclient_script=None):
    """Enable the list of network interfaces.

    Args:
      interfaces: list of string, the output device names to enable.
      logger: logger object, used to write to SysLog and serial port.
      dhclient_script: string, the path to a dhclient script used by dhclient.
    """
    interfaces_to_up = [i for i in interfaces if i != 'eth0']
    if interfaces_to_up:
      logger.info('Enabling the Ethernet interfaces %s.', interfaces_to_up)
      self._WriteIfcfg(interfaces_to_up, logger)
      self._Ifup(interfaces_to_up, logger)

  def _WriteIfcfg(self, interfaces, logger):
    """Write ifcfg files for multi-NIC support.

    Overwrites the files. This allows us to update ifcfg-* in the future.
    Disable the network setup to override this behavior and customize the
    configurations.

    Args:
      interfaces: list of string, the output device names to enable.
      logger: logger object, used to write to SysLog and serial port.
    """
    for interface in interfaces:
      interface_config = os.path.join(
          self.network_path, 'ifcfg-%s' % interface)
      interface_content = [
          '# Added by Google.',
          'STARTMODE=hotplug',
          'BOOTPROTO=dhcp',
          'DHCLIENT_SET_DEFAULT_ROUTE=yes',
          'DHCLIENT_ROUTE_PRIORITY=10%s00' % interface,
          '',
      ]
      with open(interface_config, 'w') as interface_file:
        interface_file.write('\n'.join(interface_content))
      logger.info('Created ifcfg file for interface %s.', interface)

  def _Ifup(self, interfaces, logger):
    """Activate network interfaces.

    Args:
      interfaces: list of string, the output device names to enable.
      logger: logger object, used to write to SysLog and serial port.
    """
    ifup = ['/usr/sbin/wicked', 'ifup', '--timeout', '1']
    try:
      subprocess.check_call(ifup + interfaces)
    except subprocess.CalledProcessError:
      logger.warning('Could not activate interfaces %s.', interfaces)

  def HandleClockSync(self, logger):
    """Sync the software clock with the hypervisor clock.

    Args:
      logger: logger object, used to write to SysLog and serial port.
    """
    helpers.CallHwclock(logger)

  def IpForwardingUtils(self, logger, proto_id=None):
    """Get system IP address configuration utilities.

    Args:
      logger: logger object, used to write to SysLog and serial port.
      proto_id: string, the routing protocol identifier for Google IP changes.
    """
    return ip_forwarding_utils.IpForwardingUtilsIproute(logger, proto_id)
